/* ADS8329.h */
/* K. Schneider */
/* EDC */
/* Driver for ADS8329 */

#ifndef _ADS8329_H_
#define _ADS8329_H_

//#include "error_codes.h"
//#include "hardware.h"

//Min. 500, max. 1485 und durch Int 200ns Verzgerung, Trigger muss min 40ns auf L sein (6 CPU Takte)
#define ADS8329_SET_TRIGGER(CMPWERT)		EPwm2Regs.CMPB=CMPWERT

// Command Set Defined by Command Register (CMR)

// D[15:12]
#define ADS8329_CMR_IPC_0			0x0000	///< CMR-ADS8329 Select analog input channel 0
#define ADS8329_CMR_IPC_1			0x1000	///< CMR-ADS8329 Select analog input channel 1
#define ADS8329_CMR_RESERVED_2		0x2000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_3		0x3000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_4		0x4000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_5		0x5000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_6		0x6000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_7		0x7000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_8		0x8000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_9		0x9000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_RESERVED_A		0xA000	///< CMR-ADS8329 Reserved
#define ADS8329_CMR_WAKE_UP			0xB000	///< CMR-ADS8329 Wake up
#define ADS8329_CMR_READ_CFR		0xC000	///< CMR-ADS8329 Read CFR
#define ADS8329_CMR_READ_DATA		0xD000	///< CMR-ADS8329 Read Data
#define ADS8329_CMR_WRITE_CFR		0xE000	///< CMR-ADS8329 Write CFR
#define ADS8329_CMR_DEFAULT_MODE	0xF000	///< CMR-ADS8329 Default mode (load CFR with default value)

// Configuration Register (CFR) Map

// Channel select mode
// D11
#define ADS8329_CFR_MANUAL_CHANNEL				0x0000	///< CFR-ADS8329 Manual channel select enabled
#define ADS8329_CFR_AUTO_CHANNEL				0x0800	///< CFR-ADS8329 Auto channel select enabled

// Conversion clock (CCLK) source select
// D10
#define ADS8329_CFR_SCLK_DIV2					0x0000	///< CFR-ADS8329 Conversion clock (CCLK) = SCLK/2
#define ADS8329_CFR_INTERNAL_OSC				0x0400	///< CFR-ADS8329 Conversion clock (CCLK) = Internal OSC

// Trigger (conversion start) select: start conversion at the end of sampling (EOS). If D9 = 0, the D4 setting is ignored.
// D9
#define ADS8329_CFR_AUTO_TRIGGER				0x0000	///< CFR-ADS8329 Auto trigger automatically starts (4 internal clocks after EOC inactive)
#define ADS8329_CFR_MANUAL_TRIGGER				0x0200	///< CFR-ADS8329 Manual trigger manually started by falling edge of /CONVST

// Don't care
// D8

// Pin 10 polarity select when used as an output (EOC/INT)
// D7
#define ADS8329_CFR_EOC_INT_ACTIV_HIGH			0x0000	///< CFR-ADS8329 EOC Active high / /INT active high
#define ADS8329_CFR_EOC_INT_ACTIV_LOW			0x0080	///< CFR-ADS8329 EOC Active low / /INT active low

// Pin 10 function select when used as an output (EOC/INT)
// D6
#define ADS8329_CFR_PIN_INT						0x0000	///< CFR-ADS8329 Pin used as /INT
#define ADS8329_CFR_PIN_EOC						0x0040	///< CFR-ADS8329 Pin used as EOC

// Pin 10 I/O select for chain mode operation
// D5
#define ADS8329_CFR_PIN_CDI_INPUT				0x0000	///< CFR-ADS8329 Pin 10 is used as CDI input (chain mode enabled)
#define ADS8329_CFR_PIN_EOC_INT_OUTPUT			0x0020	///< CFR-ADS8329 Pin 10 is used as EOC / /INT output

// Auto nap power-down enable/disable (mid voltage and comparator shut down between cycles). This bit setting is ignored if D9 = 0.
// D4
#define ADS8329_CFR_AT_NP_PWR_DOWN_ENABLED		0x0000	///< CFR-ADS8329 Auto nap power-down enabled (not activated)
#define ADS8329_CFR_AT_NP_PWR_DOWN_DISABLED		0x0010	///< CFR-ADS8329 Auto nap power-down disabled

// Nap power-down (mid voltage and comparator shut down between cycles). This bit is set to 1 automatically by wake-up command.
// D3
#define ADS8329_CFR_EN_DEV_NAP_PWR_DOWN			0x0000	///< CFR-ADS8329 Auto nap power-down enabled (not activated)
#define ADS8329_CFR_DIS_DEV_NAP_PWR_DOWN		0x0008	///< CFR-ADS8329 Auto nap power-down disabled

// Deep power-down. This bit is set to 1 automatically by wake-up command.
//D2
#define ADS8329_CFR_EN_DEV_DEEP_PWR_DOWN		0x0000	///< CFR-ADS8329 Enable/activate device in deep power-down
#define ADS8329_CFR_DIS_DEV_DEEP_PWR_DOWN		0x0004	///< CFR-ADS8329 Remove device from deep power-down (resume)

// TAG bit enable. This bit is ignored by the ADS8329 and is always read 0
// D1
#define ADS8329_CFR_DIS_TAG_BIT					0x0000	///< CFR-ADS8329 TAG bit disabled.
#define ADS8329_CFR_EN_TAG_BIT					0x0002	///< CFR-ADS8329 TAG bit output enabled.

// Reset
// D0
#define ADS8329_CFR_SYSTEM_RESET				0x0000	///< CFR-ADS8329 System reset
#define ADS8329_CFR_NORMAL_OPERATION			0x0001	///< CFR-ADS8329 Normal operation

#define ADS8329_DEV_1	1
#define ADS8329_DEV_2	2
#define ADS8329_DEV_3	3


#define CURRENT_MEASURE_TIMES_BETWEEN_FOC	FOC_BERECHNUNG_ALLE_X_MAL

#define CURRENT_MEASURE_BOTH				1
#define CURRENT_MEASURE_UP					0
#define CURRENT_MEASURE_DOWN				2

#define CURRENT_MEASURE_UP_DOWN				CURRENT_MEASURE_UP

/* ----------------------------------------------------------------------------
 *			ADS8329 Anschlusse an McBSP-Modul
 * --------------------------------------------------------------------------*/
//CS-Leitungen
#define	ADS1_CS_L()							GpioDataRegs.GPACLEAR.bit.GPIO15 = 1
#define	ADS1_CS_H()							GpioDataRegs.GPASET.bit.GPIO15 = 1

#define	ADS23_CS_L()						GpioDataRegs.GPACLEAR.bit.GPIO27 = 1
#define	ADS23_CS_H()						GpioDataRegs.GPASET.bit.GPIO27 = 1


#if((CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_UP) || (CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH))
struct MOTORCURR_SEP {
	unsigned int current3_measure1_up;
	unsigned int current2_measure1_up;
	unsigned int current1_measure1_up;
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH
	unsigned int current3_measure1_down;
	unsigned int current2_measure1_down;
	unsigned int current1_measure1_down;
#endif
	
#if CURRENT_MEASURE_TIMES_BETWEEN_FOC > 1
	unsigned int current3_measure2_up;
	unsigned int current2_measure2_up;
	unsigned int current1_measure2_up;
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH
	unsigned int current3_measure2_down;
	unsigned int current2_measure2_down;
	unsigned int current1_measure2_down;
#endif
#endif

#if CURRENT_MEASURE_TIMES_BETWEEN_FOC > 2
	unsigned int current3_measure3_up;
	unsigned int current2_measure3_up;
	unsigned int current1_measure3_up;
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH
	unsigned int current3_measure3_down;
	unsigned int current2_measure3_down;
	unsigned int current1_measure3_down;
#endif
#endif

};

struct MOTORCURR_ARRAY{
	unsigned int					array[CURRENT_MEASURE_TIMES_BETWEEN_FOC*(CURRENT_MEASURE_UP_DOWN+1)*3];
};

#else
struct MOTORCURR_SEP {
	unsigned int current3_measure1_down;
	unsigned int current2_measure1_down;
	unsigned int current1_measure1_down;

#if CURRENT_MEASURE_TIMES_BETWEEN_FOC > 1
	unsigned int current3_measure2_down;
	unsigned int current2_measure2_down;
	unsigned int current1_measure2_down;
#endif

#if CURRENT_MEASURE_TIMES_BETWEEN_FOC > 2
	unsigned int current3_measure3_down;
	unsigned int current2_measure3_down;
	unsigned int current1_measure3_down;
#endif

};

struct MOTORCURR_ARRAY{
	unsigned int					array[CURRENT_MEASURE_TIMES_BETWEEN_FOC*3];
};

#endif

union MOTORCURRENT_CONT{
	struct MOTORCURR_ARRAY			all;
	struct MOTORCURR_SEP			separate;
};

extern union MOTORCURRENT_CONT motor_current;

typedef struct {
	unsigned int ConfigReg1;
	unsigned int ConfigReg2;
	unsigned int ConfigReg3;
} ADS8329ConfigRegister_cont;

//*****************************************************************************
/*
 *	Funktionen fr den externen ADCs
 */
//*****************************************************************************
extern Uint16 ADCInit();
//extern void ADCConversionStart();
//extern void ADCGetMotorCurrent();
//extern void ADCReceiveSampleData(MotorCurr_cont *p_MeasCurr);
extern void ADS8329ReadCFRDaisyChain(ADS8329ConfigRegister_cont* p_CfgReg);
void ADS8329Write(unsigned int data, unsigned char dev);
volatile Uint16* PtrADS8329DataRec(void);

//*****************************************************************************
/*
 *	Funktionen fr den McBSP-Port B
 */
//*****************************************************************************
extern void McbspBInit(void);
extern void McbspBInitDaisyChain(void);
extern void McbspBWrite16(unsigned int data);
extern unsigned int McbspBRead16(void);
void InitMcbspbGpio(void);

//*****************************************************************************
/*
 *	Funktionen fr den DMA
 */
//*****************************************************************************
void DmaCh5Ch6InitAds8329(void);
void DmaCh5Ch6StartAds8329(void);

#endif /* _MCBSP_B_H_ */
