#ifndef EPWM_H_
#define EPWM_H_

/*!	\def EPWM_COUNTER_PERIOD
 * 	\brief	Berechnung des maximalen Wertes des PWM-Zhlers
 * 
 * 	Die PWM-Einheit luft mit der Frequenz der CPU und im Up-Down-Modus,
 * 	wodurch sich die 2 ergibt.
 */
#define	EPWM_COUNTER_PERIOD					((unsigned int)(CPU_FRQ/EPWM_FRQ/2))

/*!	\def PWM_TOTZEIT
 * 	\brief Berechnung der Totzeit
 * 
 * 	Die Totzeit kann im file [@ref global.h] mittels der Definition PWM_TOTZEIT_NS
 * 	in ns angegeben werden. Die Umrechnung erfolgt in gleicher Weise wie der
 * 	maximale Wert des PWM-Zhlers.
 */
#define	PWM_TOTZEIT				((unsigned int)((double)PWM_TOTZEIT_NS*(double)CPU_FRQ/1000000000.0))

/*!	\def PWM_TOTZEIT_2
 * 	\brief Berechnung der halben Todzeit
 * 
 * 	Diese Berechnung erfolgt wie bei TODZEIT nur fr die Hlfte. Die Berechnung
 * 	erfolgt neu, weil sie dadurch genauer ist und die Umwandlung in int erst
 * 	nach der Berechnung erfolgt.
 *	actual: PWM_TOTZEIT_2 = 9, see epwm.h
 */
#define	PWM_TOTZEIT_2			((unsigned int)((double)PWM_TOTZEIT120_NS*(double)CPU_FRQ/1000000000.0/2.0))
#define	PWM_TOTZEIT70NS_2			((unsigned int)((double)PWM_TOTZEIT70_NS*(double)CPU_FRQ/1000000000.0/2.0))
#define	PWM_TOTZEIT120NS_2			((unsigned int)((double)PWM_TOTZEIT120_NS*(double)CPU_FRQ/1000000000.0/2.0))

//*****************************************************************************
/*
 * 	Funktionen
 */
//*****************************************************************************
void StartFOC(void);
void InitEPwmModul(void);
volatile Uint16* PtrEpwmErrCnt(void);
void PWMDisable(void);
void PWMEnable(void);
void TriggerGenEnable(void);
void TriggerGenDisable(void);
void InitEPwm1(void);
void InitEPwm2(void);
void InitEPwm3(void);
void InitEPwm4(void);
void InitEPwm5(void);
void InitEPwm6(void);
void InitEPwmGpio(void);
#endif /*EPWM_H_*/
