///////////////////////////////////////////////////////////////////////////
//            COPYRIGHT (c) 2010 GEMAC mbH                               //
//            Software und Quelltext sind Eigentum der                   //
//                                                                       //
// 	                      G E M A C  m b H                               //
//                                                                       //
//         und unterstehen den beigefgten Lizenzbestimmungen.           //
//                                                                       //
//                                                                       //
//         GEMAC mbH                                                     //
// 		   ZWICKAUER STR.227 / 09116 CHEMNITZ / GERMANY                  //
//         TEL. (+49)371/3377-305 / FAX. (+49)371/3377-272               //
// 		   www.gemac-chemnitz.de                                         //
///////////////////////////////////////////////////////////////////////////
///
/// \file		gcip2000.h
/// \brief 		Defines GC-IP2000
/// \author		friese
///	History
/// 	21.10.2010	File created
///
///
///	Last modification	: $Date: 2010-11-24 09:26:00 +0100 (Mi, 24. Nov 2010) $
/// Modified by			: $Author: friese $
/// File revision		: $Rev: 7430 $
///
/// \details
///
////////////////////////////////////////////////////////////////////////////
#ifndef _GCIP2000_H_
#define _GCIP2000_H_

// DATATYPES
//typedef unsigned int             WORD;
//typedef unsigned long            DWORD;


// OPCODES
#define GCIP_OPC_WRA 			0x8				///< OPC-GC-IP2000 Write Adress
#define GCIP_OPC_WRD 			0xA				///< OPC-GC-IP2000 Write Data
#define GCIP_OPC_RD0 			0xC				///< OPC-GC-IP2000 Store/Readout LSB
#define GCIP_OPC_RD1 			0xE				///< OPC-GC-IP2000 Readout MSB
#define GCIP_OPC_NOP 			0x0				///< OPC-GC-IP2000 No operation, output only
#define GCIP_NBRDCST			0x1				///< OPC-GC-IP2000 Broadcast if 0

//Hardware Adresse
#define GCIP_OPC_HWA			0x0			///< OPC-GC-IP2000 Hardware Adress

//Register
#define GCIP_ADR_MVAL   	 	0x00			///< REG-GC-IP2000 Measurement value
#define GCIP_ADR_CNT    		0x04			///< REG-GC-IP2000 Counter
#define GCIP_ADR_STAT   	 	0x08			///< REG-GC-IP2000 Status
#define GCIP_ADR_CMD     	 	0x0B			///< REG-GC-IP2000 Command
#define GCIP_ADR_CFG1    		0x0C			///< REG-GC-IP2000 Config 1
#define GCIP_ADR_CFG1_0			0x0C			///< REG-GC-IP2000 Config 1
#define GCIP_ADR_CFG1_1			0x0D			///< REG-GC-IP2000 Config 1
#define GCIP_ADR_CFG1_2			0x0E			///< REG-GC-IP2000 Config 1
#define GCIP_ADR_CFG1_3			0x0F			///< REG-GC-IP2000 Config 1
#define GCIP_ADR_CFG2			0x10			///< REG-GC-IP2000 Config 2
#define GCIP_ADR_CFG2_0			0x10			///< REG-GC-IP2000 Config 2
#define GCIP_ADR_CFG2_1			0x11			///< REG-GC-IP2000 Config 2
#define GCIP_ADR_CFG2_2			0x12			///< REG-GC-IP2000 Config 2
#define GCIP_ADR_CFG2_3			0x13			///< REG-GC-IP2000 Config 2
#define GCIP_ADR_CNTRLG			0x14			///< REG-GC-IP2000 Gain-Control
#define GCIP_ADR_CNTRLGC_0		0x14			///< REG-GC-IP2000 Gain-Control
#define GCIP_ADR_CNTRLGC_1		0x15			///< REG-GC-IP2000 Gain-Control
#define GCIP_ADR_CNTRLGS_0		0x16			///< REG-GC-IP2000 Gain-Control
#define GCIP_ADR_CNTRLGS_1		0x17			///< REG-GC-IP2000 Gain-Control
#define GCIP_ADR_CNTRLO			0x18			///< REG-GC-IP2000 Offset-Control
#define GCIP_ADR_CNTRLOC_0		0x18			///< REG-GC-IP2000 Offset-Control
#define GCIP_ADR_CNTRLOC_1		0x19			///< REG-GC-IP2000 Offset-Control
#define GCIP_ADR_CNTRLOS_0		0x1A			///< REG-GC-IP2000 Offset-Control
#define GCIP_ADR_CNTRLOS_1		0x1B			///< REG-GC-IP2000 Offset-Control
#define GCIP_ADR_RSV			0x1C			///< REG-GC-IP2000 Do not change!
#define GCIP_ADR_EEP			0x20			///< REG-GC-IP2000 EEPROM
#define GCIP_ADR_EEPDAT			0x20			///< REG-GC-IP2000 EEPROM Write Data
#define GCIP_ADR_EEPADR			0x21			///< REG-GC-IP2000 EEPROM Adr
#define GCIP_ADR_EEPOPC			0x22			///< REG-GC-IP2000 EEPROM Opcode
#define GCIP_ADR_ADC			0x24			///< REG-GC-IP2000 ADC
#define GCIP_ADR_CADC			0x28			///< REG-GC-IP2000 ADC after correction
#define GCIP_ADR_IPOL1			0x2C			///< REG-GC-IP2000 interpolation result 1
#define GCIP_ADR_IPOL2			0x30			///< REG-GC-IP2000 interpolation result 2

#define GCIP_CMD_RESCNT			0x01			///< Command-Register: Clear Counter
#define GCIP_CMD_RESCTRL		0x02			///< Command-Register: Reset Gain/Offset-Control
#define GCIP_CMD_CLRZ			0x04			///< Command-Register: Reset ZSTAT-Bit

//#pragma pack(1)


/// \brief SPI-Word Low-level
typedef union _UNION_GCIPSPI
{
	Uint16 dwVal32;			///< Register-value DWORD
	struct				///< Register-value 32 bit
	{
		Uint16 val : 8;		///< adress or data
		Uint16 hwa : 4;		///< Hardwareadress
		Uint16 opc : 4;		///< opcode and broadcast-bit
	} bit;
} UNION_GCIPSPI;

/// \brief SPI-Word Low-level (32 Bit filled)
typedef union _UNION_GCIPSPI_32
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 val : 8;		///< adress or data
		Uint32 hwa : 4;		///< Hardwareadress
		Uint32 opc : 4;		///< opcode and broadcast-bit
		Uint32 rsv : 16;		///< gap for 32 bit
	} bit;
} UNION_GCIPSPI_32;

/// \brief Interpolation rates
typedef enum _ENUM_GCIP_IRATE
{
	IRATE_2000 = 0,			///<Interpolation rate 2000
	IRATE_1600,				///<Interpolation rate 1600
	IRATE_1000,				///<Interpolation rate 1000
	IRATE_800,				///<Interpolation rate 800
	IRATE_500,				///<Interpolation rate 500
	IRATE_400,				///<Interpolation rate 400
	IRATE_200,				///<Interpolation rate 200
	IRATE_100,				///<Interpolation rate 100
	IRATE_2048,				///<Interpolation rate 2048
	IRATE_1024,				///<Interpolation rate 1024
	IRATE_512,				///<Interpolation rate 512
	IRATE_256,				///<Interpolation rate 256
	IRATE_128				///<Interpolation rate 128
} ENUM_GCIP_IRATE;

/// \brief Nominal Amplitude of analog sensor
typedef enum _ENUM_GCIP_INPUT
{
	INPUT_1Vpp = 0,			///<1Vpp
	INPUT_500mVpp,			///<500mVpp
	INPUT_250mVpp,			///<250mVpp
	INPUT_80mVpp			///<80mVpp
} ENUM_GCIP_INPUT;

/// \brief Configuration GC-IP2000: CFG1
typedef union _UNION_GCIP_CFG1
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 irate	: 4;	///< interpolation rate
		Uint32 Z4  	: 1;	///< unused
		Uint32 DISZ 	: 1;	///< deactivate reference
		Uint32 mode	: 2;	///< DO NOT CHANGE!!
		Uint32 tpp	: 3;	///< unused
		Uint32 dflt	: 2;	///< unused
		Uint32 trgslp: 1;	///< DO NOT CHANGE!!
		Uint32 gain	: 2;	///< input voltage
		Uint32 emask	: 6;	///< error processing mask
		Uint32 hld	: 1;	///< unused
		Uint32 ledm	: 1;	///< unused
		Uint32 lmask	: 6;	///< error processing storage
		Uint32 lpf	: 1;	///< input filter analog
		Uint32 rsv	: 1;	///< unused
	} bit;
} UNION_GCIP_CFG1;
#define GCIP_CFG1_DEFAULT		0x77370050		///< Default CFG1: IR=2000, DISZ, MODE1, MABZ=0, LABZ=0, LPF=1, 1Vpp

/// \brief Configuration GC-IP2000: CFG2
typedef union _UNION_GCIP_CFG2
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 timer	: 10;	///< DO NOT CHANGE!!
		Uint32 sync 	: 5;	///< Must fit to Loop-Mem
		Uint32 async	: 1;	///< Must fit to Loop-Mem
		Uint32 disctl: 1;	///< Deactivate control
		Uint32 gctl	: 2;	///< Controlparameter gain
		Uint32 octl	: 2;	///< Controlparameter offset
		Uint32 rsv	: 3;	///< unused
		Uint32 phase	: 6;	///< phase value
		Uint32 phber	: 1;	///< phase range
		Uint32 rsv2	: 1;	///< unused
	} bit;
} UNION_GCIP_CFG2;
#define GCIP_CFG2_DEFAULT		0x00008000		///< Default CFG2: ASYNC

/// \brief Measurement GC-IP2000: MVAL
typedef union _UNION_GCIP_MVAL
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 err	: 1;	///< Error
		Uint32 trg 	: 1;	///< source trigger if true, counter otherwise
		Uint32 count	: 30;	///< Counter value
	} bit;
} UNION_GCIP_MVAL;

/// \brief Measurement GC-IP2000: Count
typedef union _UNION_GCIP_CNT
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 err	: 1;	///< Error
		Uint32 zstat : 1;	///< counter belongs to reference if true
		Uint32 count	: 30;	///< Counter value
	} bit;
} UNION_GCIP_CNT;

/// \brief Measurement GC-IP2000: STAT
typedef union _UNION_GCIP_STAT
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 evlow : 1;	///< Error - Amp. Sin & Cos
		Uint32 ecadc : 1;	///< Error - ADC Cos
		Uint32 esadc : 1;	///< Error - ADC Sin
		Uint32 efast : 1;	///< Error - speed
		Uint32 eabz  : 1;	///< Error - speed ABZ
		Uint32 ecgain: 1;	///< Error - Gain Cos
		Uint32 esgain: 1;	///< Error - Gain Sin
		Uint32 ecoff : 1;	///< Error - Offset Cos
		Uint32 esoff : 1;	///< Error - Offset Sin
		Uint32 trg	: 1;	///< next mval from trigger
		Uint32 tim	: 1;	///< next mval from timer
		Uint32 ovl	: 1;	///< trigger lost
		Uint32 zstat	: 1;	///< counter belongs to reference
		Uint32 led	: 2;	///< level of led-pin
		Uint32 rsv1	: 1;	///< unused
		Uint32 rev	: 4;	///< ASIC revision
		Uint32 id	: 4;	///< ASIC ID
		Uint32 rsv2	: 8;	///< unused
	} bit;
} UNION_GCIP_STAT;

/// \brief Measurement GC-IP2000: Gain-Control
typedef union _UNION_GCIP_CTRLG
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 gainc : 11;	///< Gain correction Cos
		Uint32 rsv1	: 5;	///< unused
		Uint32 gains : 11;	///< Gain correction Sin
		Uint32 rsv2	: 5;	///< unused
	} bit;
} UNION_GCIP_CTRLG;

/// \brief Measurement GC-IP2000: Gain-Control
typedef union _UNION_GCIP_CTRLO
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 offc : 13;	///< Offset correction Cos
		Uint32 rsv1	: 3;	///< unused
		Uint32 offs : 13;	///< Offset correction Sin
		Uint32 rsv2	: 3;	///< unused
	} bit;
} UNION_GCIP_CTRLO;

/// \brief Measurement GC-IP2000: ADC
typedef union _UNION_GCIP_ADC
{
	Uint32 dwVal32;			///< Register-value DWORD
	Uint16 wVal[2];			///< Register-value 2 x WORD
	struct 				///< Register-value 32 bit
	{
		Uint32 adcc : 16;	///< ADC Cos
		Uint32 adcs : 16;	///< ADC Sin
	} bit;
} UNION_GCIP_ADC;

/// \brief Measurement GC-IP2000: ADC after correction
typedef union _UNION_GCIP_CADC
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 absc : 15;	///< absolute value cos
		Uint32 sigc : 1;		///< sign cos
		Uint32 abss : 15;	///< absolute value sin
		Uint32 sigs : 1;		///< sign sin
	} bit;
} UNION_GCIP_CADC;

/// \brief Measurement GC-IP2000: PHI/DPHI
typedef union _UNION_GCIP_IP1
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 phi  : 16;	///< phase position sin/cos
		Uint32 dphi : 16;	///< difference of phase position sin/cos between samples
	} bit;
} UNION_GCIP_IP1;

/// \brief Measurement GC-IP2000: PHI/DPHI
typedef union _UNION_GCIP_IP2
{
	Uint32 dwVal32;			///< Register-value DWORD
	struct 				///< Register-value 32 bit
	{
		Uint32 phi  : 16;	///< phase position sin/cos
		Uint32 bq   : 16;	///< vector lenght sin/cos
	} bit;
} UNION_GCIP_IP2;

struct GCIP2000REG_CONT{
	union _UNION_GCIPSPI			spi;
	union _UNION_GCIP_CFG1			cfg1;
	union _UNION_GCIP_CFG2			cfg2;
	union _UNION_GCIP_MVAL			mval;
	union _UNION_GCIP_CNT			cnt;
	union _UNION_GCIP_STAT			stat;
	union _UNION_GCIP_CTRLG			ctrlg;
	union _UNION_GCIP_CTRLO			ctrlo;
	union _UNION_GCIP_ADC			adc;
	union _UNION_GCIP_CADC			cadc;
	union _UNION_GCIP_IP1			ip1;
	union _UNION_GCIP_IP2			ip2;
};

extern struct GCIP2000REG_CONT gcip2000;


//#pragma pack()

#endif /* _GCIP2000_H_ */
