/******************************************************************************
 * Firma             :  Thermo Electron (Karlsruhe) GmbH                      *
 * BU                :  Material Characterization Business                    *
 * Erstelldatum      :  31-Jul-2002                                           *
 * Letzte Aenderung  :  18-Mar-2004                                           *
 * Projekt           :  alle                                                  *
 * Verwendungszweck  :  Die Datei enthalt die Funktion fuer die automatische  *
 *                      Bestimmung von prozessorabhaengigen Konstanten.       *
 * Referenzdokumente :                                                        *
 * Autor             :  Uwe Bauer                                             *
 * Tester            :  Uwe Bauer                                             *
 * Teststatus        :  draft                                                 *
 * Editor            :  Microsoft Visual C++ 6.0                              *
 * Compiler          :  Microsoft Visual C++ 6.0                              *
 * Versionsnummer    :  V 1.00                                                *
 * Revision History  :                                                        *
 ******************************************************************************/

/*
 * Um wiederholtes Parsen von Quellcode zu vermeiden wird jede Header - Datei
 * gekapselt. Alle Deklarationen sollten zwischen den Pre - Prozessor - 
 * Anweisungen stehen.
 */
#if !defined(MYCONST_H)          /* Beginn der Kapselung von MYCONST_H        */
#define MYCONST_H

#define TMS320F28335

/*
 * Fuer die zugehoerige Quellcodedatei werden alle Variablen und Funktioen
 * als intern und fuer alle anderen Quellcodedateien entsprechend als extern
 * deklariert.
 */
#undef	VAR_TYPE	
#undef	FNCT_TYPE	

#if defined(MYCONST_C)

#define	VAR_TYPE
#define	FNCT_TYPE

#else

#define  VAR_TYPE    extern
#define  FNCT_TYPE   extern

#endif



/******************************************************************************
 * Makros.                                                                    *
 ******************************************************************************/



/******************************************************************************
 * An dieser Stelle des Programmes erfolgt die Deklaration von benutzer-      *
 * definierten Datentypen.                                                    *
 ******************************************************************************/



/******************************************************************************
 * An dieser Stelle des Prgrammes erfolgt die Deklaration von Variablen. Dabei*
 * werden zuerst die globalen und dann die lokalen Variablen deklariert.      *
 ******************************************************************************/



/******************************************************************************
 * An dieser Stelle des Programmes erfolgt die Deklaration von globalen       *
 * Initialisierungsroutinen.                                                  *
 ******************************************************************************/



/******************************************************************************
 * An dieser Stelle des Programmes erfolgt die Deklaration aller anderen      *
 * globalen Funktionen. Die Beschreibung von extern sichtbaren Funktionen     *
 * erfolgt am Ort der Deklaration. Bei einer selbst erstellten Bibliothek     *
 * wird dem Benutzer der Bibliothek nur die Header - Datei als einziger Teil  *
 * der Quelldatei zugaenglich gemacht.                                        *
 ******************************************************************************/



/******************************************************************************
 * Funktion       :  NumberBasis()                                            *
 * Datum          :  31-Jul-2002                                              *
 * Letze Aenderung:  18-Mar-2004                                              *
 * Autor          :  Uwe Bauer                                                *
 * -------------------------------------------------------------------------- *
 * Kurze Beschreibung:                                                        *
 * ==================                                                         *
 * Mit der Funktion wird die Zahlenbasis des verwendeten Compilers und der    *
 * Zielhardware ermittelt.                                                    *
 *                                                                            *
 * Argumente:                                                                 *
 * =========                                                                  *
 *    keine                                                                   *
 *                                                                            *
 * Rueckgabewert:                                                             *
 * =============                                                              *
 *    Zahlenbasis                                                             *
 ******************************************************************************/
FNCT_TYPE Uint16 NumberBasis(void);

/******************************************************************************
 * Funktion       :  RelAccuracy()                                            *
 * Datum          :  31-Jul-2002                                              *
 * Letze Aenderung:  18-Mar-2004                                              *
 * Autor          :  Uwe Bauer                                                *
 * -------------------------------------------------------------------------- *
 * Kurze Beschreibung:                                                        *
 * ==================                                                         *
 * Mit der Funktion wird die relative Zahlengenauigkeit der Gleitpunkt-       *
 * zahlendarstellung des verwendeten Compilers und der Zielhardware ermittelt.*
 * EPS gibt die Entfernung von 1.0 zur naechst groesseren Gleitpunktzahl an.  *
 *                                                                            *
 * Argumente:                                                                 *
 * =========                                                                  *
 *    keine                                                                   *
 *                                                                            *
 * Rueckgabewert:                                                             *
 * =============                                                              *
 *    relative Zahlengenauigkeit                                              *
 ******************************************************************************/
FNCT_TYPE float64 RelAccuracy(void);

/******************************************************************************
 * Funktion       :  MaxWord()                                                *
 * Datum          :  31-Jul-2002                                              *
 * Letze Aenderung:  18-Mar-2004                                              *
 * Autor          :  Uwe Bauer                                                *
 * -------------------------------------------------------------------------- *
 * Kurze Beschreibung:                                                        *
 * ==================                                                         *
 * Mit der Funktion wird der Wertebereich der boolschen Wortvariable bestimmt.*
 * MAXWORD wird z.B. als maximal zulaessiges Gedaechtnis beim ANW-MWB         *
 * verwendet.                                                                 *
 *                                                                            *
 * Argumente:                                                                 *
 * =========                                                                  *
 *    keine                                                                   *
 *                                                                            *
 * Rueckgabewert:                                                             *
 * =============                                                              *
 *    groesste darstellbare Zahl im Wortformat                                *
 ******************************************************************************/
FNCT_TYPE Uint16 MaxWord(void);

/******************************************************************************
 * Funktion       :  MaxByte()                                                *
 * Datum          :  31-Jul-2002                                              *
 * Letze Aenderung:  18-Mar-2004                                              *
 * Autor          :  Uwe Bauer                                                *
 * -------------------------------------------------------------------------- *
 * Kurze Beschreibung:                                                        *
 * ==================                                                         *
 * Mit der Funktion wird der Wertebereich der boolschen Bytevariable bestimmt.*
 *                                                                            *
 * Argumente:                                                                 *
 * =========                                                                  *
 *    keine                                                                   *
 *                                                                            *
 * Rueckgabewert:                                                             *
 * =============                                                              *
 *    groesste darstellbare Zahl im Byteformat                                *
 ******************************************************************************/
FNCT_TYPE Uint8 MaxByte(void);



/******************************************************************************
 * An dieser Stelle des Programmes erfolgt die Deklaration von globalen       *
 * Konstanten.                                                                *
 ******************************************************************************/

#if defined(TMS320F28335)

/*
 * IEEE - Zahlenformat
 */
#define  NUMBERBASIS 2                       /* Zahlenbasis                   */
#define  EPS         2.22044604925031e-016   /* relative Zahlengenauigkeit    */
#define  MAXWORD     65535                   /* groesste Wortvariable         */
#define  MAXBYTE     255                     /* groesste Bytevariable         */

#else

/*
 * Zahlenformate automatisch ermitteln
 */

#define  NUMBERBASIS NumberBasis()           /* Zahlenbasis                   */
#define  EPS         RelAccuracy()           /* relative Zahlengenauigkeit    */
#define  MAXWORD     MaxWord()               /* groesste Wortvariable         */
#define  MAXBYTE     MaxByte()               /* groesste Bytevariable         */

#endif

/*
 * boolsche Konstanten
 */

#if !defined(TRUE)
#define  TRUE ((Uint16) 1)                   /* wahr                          */
#endif

#if !defined(FALSE)
#define  FALSE ((Uint16) 0)                  /* falsch                        */
#endif

#if !defined(OK)
#define  OK    ((Uint16) 1)                  /* Zustand in Ordnung            */
#endif

#if !defined(NOK)
#define  NOK   ((Uint16) 0)                  /* Zustand nicht in Ordnung      */
#endif

#if !defined(ON)
#define  ON    ((Uint16) 1)                  /* einschalten                   */
#endif

#if !defined(OFF)
#define  OFF   ((Uint16) 0)                  /* ausschalten                   */
#endif



#endif   /* Ende der Kapselung von MYCONST_H */ 
