/*!
\file   Uc1dsp.h
\brief  Vereinbarungen Kommunikation C-1 <--> Dsp-1 Control

\author		Koester
\date		01.10.2012

changes:	09.12.15 Motortemperatur vom C-1 hinzu

*/

// Schalter fr bersetzung (Stellaris oder DSP TMS320F28335)
//#define  COMP_STELLARIS
#define  COMP_DSP

/* Include files ************************************************************/

#ifdef TODO

#ifdef   COMP_DSP
#include "typedefs.h"
#endif
#ifdef  COMP_XC167
#include "mytypes.h"
#endif

#endif

/* Functions  ************************************************************/

/* Constants *************************************************************/

/* Macro definitions *****************************************************/

// C167-Treibertypen Status
#define NONE        0
#define CSOSC       1
#define CDOSC       2
#define MWOSC       3
#define CRROT       4
#define CSROT       5
#define CCROT       6 	// Cal.-Mode
#define CDROT       7
#define CSFASTOSC	8

// Macros for different Data-Modes in tMeas.tSet....
#define RMP_OFF 	0
#define RMP_ON_LIN	1
#define RMP_ON_LOG	2

// Macros for different Data-Modes in tMeas.tSet....
#define RESET_DATA_FILT		0
#define NORMALMODE			1
#define MSC_INIT_AQU		2
#define MSC_START_AQU		3
#define MSC_START_ANALYSE	4
#define MSC_END				5
#define FRI_INIT_AQU		6
#define FRI_START_AQU		7
#define FRI_START_ANALYSE	8
#define XXX_RESERVE2		9
#define PRESET_DATA_FILT	20
#define SETCON_DATA_FILT	21


// Macros for different Meas.-Modes in tMeas.tData.usState
#define MM_STOP				0	// Measurement is Stop, no Measurement
#define MM_NORMAL			1	// Measurement is running
#define MSC_DATA_AQU		2	// MSC-Detection, data-aquisition busy
#define MSC_DATA_AQU_RDY	3	// MSC-Detection, data-aquisition ready
#define MSC_DATA_CAL		4	// MSC-Detection, data-calculation busy
#define MSC_DATA_CAL_RDY	5	// MSC-Detection, data-calculation busy
#define FRI_DATA_AQU		7	// MSC-Detection, data-aquisition busy
#define FRI_DATA_AQU_RDY	8	// MSC-Detection, data-aquisition ready
#define RMP_DATA_OFF	 	10	// Normal Measurement, Ramp OFF
#define RMP_DATA_LIN		11	// Normal Measurement, Ramp ON linear
#define RMP_DATA_LOG		12	// Normal Measurement, Ramp ON logarithmic
	
#define MM_E_MODE			99	// Error in receive MeasMode

/* Interruptvektoren Dpram-1 vom XC167 */
#define IV_ROTINIT			100
#define IV_ROTSTOP			101
#define IV_ROTSET			102
#define IV_ROTDATA			103

#define IV_CSOSCFASTINIT	105
#define IV_CSOSCFASTAMP		106
#define IV_CSOSCFASTSTOP	107
#define IV_OSCDATARAW		108
#define IV_OSCDATAFFT		109
#define IV_CSOSCFASTFRQ		110

#define IV_START_REFSEARCH	120
#define IV_RESET_ANGLE		121

// IV for sending constants from Stellaris (master) to Delfino (Slave)
#define IV_SEND_DEVCONST		130
#define IV_SEND_MSCCONST		131
#define IV_SEND_FRICTIONCONST	132
#define IV_SEND_G1CONST			133 // Gtable to DSP
#define IV_SEND_SENSCONST		134	//
#define IV_SEND_CORRECTION_MODE	135
#define IV_SEND_G2CONST			136	// Gtable to DSP
#define IV_SEND_C4				137	// 
#define IV_SEND_C5				138	// Reserve
#define IV_SEND_C6				139


// IV for a request constants from Delfino(Slave) to Stellaris(Master)
#define IV_REQ_DEVCONST			180 // 
#define IV_REQ_MSCCONST			181	// Request data from DSP
#define IV_REQ_FRICTIONCONST	182 //
#define IV_REQ_G1CONST			183 // Gtable from DSP
#define IV_REQ_SENSCONST		184	// Reserve
#define IV_REQ_RES				185	// Reserve
#define IV_REQ_G2CONST			186	// Gtable from DSP

#define IV_MSC_START_ANALYSE	187 // Start Data-Calculation for MSC-Detection
#define IV_MSC_START_AQ			188 // 
#define IV_FRI_START_AQ			189 // Start Data-Aquisition for Friction-Detection


#define IV_SET_ANGLE_CONSTANT   195 // Set fix angle for Test use

#define IV_INIT_FOC				200 // Start Init. EC-Motorcontrol with FOC from Stellaris
#define IV_SYSTEMDATA			250


// Macros for different Constant States in t_Const, while table-transfers
#define CONST_OK			0	// 
#define CONST_E_LENGTH		1	// Table too long
#define CONST_E_IV			2	// unknown IV

// Macros for System Info Data, Float-Array
#define MAGNET_POSITION		0	// detected absolute position magnet system
#define MSC_AMPLITUDE		1	// Difference Max-Min MSC detection run!
#define OTHERS				3


/* **************************************************************************
// Master struct: 16 * 1 Word -> 8 * 4 bytes -> 32 byte
 -> 256 Bit estimated transfer time via SSI 52s!
*************************************************************************** */
//#define SSIBUFSIZE 16	// 16 Words, 32 Bytes
#define SSIBUFSIZE 32	// 32 Words, 64 Bytes

// Diagnosis und Version-data
typedef struct
{
	Uint16	usIV;		// Int-Vektor, for Systemdata always 250
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen, always 0
	Uint32	ulVersion;	// Firmware version as unsigned long
	Uint16	usDsCpu;	// state of CPU
	Uint16	usDsIp;		// state of Interpolation
	Uint16	usDsMotCon;	// state of Motor-Control
	Uint16	usDsComm;	// state of SSI, collects the number of wrong commands received
	Uint32	ulTimer0Cnt;// Timer-0 Counter
	Uint32	ulTimer2Cnt;// Timer-2 Counter
	Uint16	usU1Comm;	// IV of last Master Command
	Uint16	us6;		// free 
	Uint16	us7;		// free
	Uint16	us8;		// always 0x5a5a for controlling the validation of the data
	float32 af[8];		// 0 - Magnet position, 1 - TMC amplitude
} t_stcSystemInfo;		// 64 Bytes

// Data structure Device and Correction data for receiving and storing function
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	usN;		// n of received datas
	Uint16	usState;	// State, e.g. Error messages
	float32	af[14];		// max. six constants
}
t_Const;		// length 64 Bytes

// Data structure Correction data for enable(1)/disable(0) Correction function
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	usFri;		// Friction
	Uint16	usMsc;		// Micro-Stress-Correction
	Uint16	usG1G2;		// G-Correction Oscillation
	Uint16	usIneRot;	// Inertia Rot
	Uint16	usIneOsc;	// Inertia Oscillation
	Uint16	usCom;		// Compliance
	Uint16	usRes;		// G-Correction Oscillation
	Uint16	usKap;		// Kappa-Correction Oscillation
	Uint16	usRaw;		// Raw-Data State
	Uint16	us1;		// 
	Uint16	us2;		// 
	Uint16	us3;		// 
	Uint16	us4;		// 
	Uint16	us5;		//
    float32 fMotorTempRef;   // 09.12.15 motor temperature from C-1
	float32 af[7];		// 
}
t_CorrectionMode;		// length 64 Bytes


// Details zum Sensor
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	nType;		// Sensortyp
	Uint16	unR1;		// Reserve	
	float32 fA;			// Sensor A-Faktor
	float32 fM;			// Sensor M-Faktor
	float32 fDamping;	// Daempfung Regler
	float32	fInertia;	//. not used
	float32	f2;			//. not used
	float32	f3;			//. not used
	float32 af[8];		//
}
t_stcSensorDsp;			// 64-Bytes



// Data structure Set Data during measurement
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	usMode;		// Measuring Mode (CS, CR, CSOSC, CDOSC)
	Uint16	usRmpMode;	// Ramp mode (ON, OFF,)
	Uint16	usDataMode; // for different modes like Fr-Correction, MSC-Corr., Reset-Phi, Reference Search a.o...
	Uint16	usK1;		// 
	float32	fSetValue;	//. Set Value for actual Mode
	float32	fMotorTemp;	//. Motor-Temperature e.g. Emergency Stop
	float32	fTableLen;  //	Table length for e.g. MSC- or Friction-Detection
	float32	fNoFreq;	//	Number of frequencies for FFT-Analysation e.g. MSC- or Friction-Det.
	float32	fRampendauer;//. not used
	float32	f6;			//. not used
	float32	f7;			//. not used
	float32	f8;			//. not used
	float32 af[5];		//. not used
}
t_MeasSetVal;			// length 64 Bytes


// Data structure Meas. Data during measurement
typedef   struct
{
	Uint16	usIV;		// Vektor
	Uint16	usID;		// ID, Untergruppe dieses Vektors
	float32	fPhi;		// Angle relativ 
	float32	fPhiAbs;	// Angle absolut to Reference mark encoder
	float32	fOmega;		// speed 
	float32	fTorque;	// torque
	float32	fTorqueAvg; // torque averige value at the moment for friction detection	
	float32	f6;			// Testweise pid_iq.Fdb
	float32 f7;			// Testweise pid_iq.Out
	float32 f8;
	float32 f9;
	float32 af[5];		// nu.
	Uint16	usState;	// for messages to master, e.g. State of MSC-Detection
	Uint16	usDiagn;	// for diagnosis inf., e.g. Errors IP, Motorcontrol, Comm.
}
t_MeasData;			// length 64 Bytes


// Daten fr Betriebsarten

// Oscillation (Fast) Measuring-Data Raw
typedef   struct
{
	Uint16	usIV;			// Int-Vektor,
	Uint16	usID;			// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	usMode;
	Uint16	usTabLength;
	// Rohdaten
	float32	fTorqueRaw;
	float32	fPhiRaw;
	float32 fPhiAbs;
	// Rheologische Daten der Grundschwingung
	float32	fTorque;
	float32	fPhi;
	float32	fDelta;
	float32	fTau0;
	float32	fGamma0;
	float32	fG1;
	float32	fG2;
	float32	fEtaStern;
	float32	fEta1;
	float32	fEta2;
	float32 fK;		// length 64 Bytes
}
t_stcOscData;

// Measuring Data Oscillation (Fast)
typedef   struct
{
	Uint16	usIV;			// Int-Vektor,
	Uint16	usID;			// ID, Untergruppe dieses Vektors fr Tabellen;
	// Amplituden aus FFT fr 1 Frequenz
	float32	afTorqueFFT[2];
	float32	afPhiFFT[2];
	float32	afDeltaRFFT[1];
	float32 af[10];
}
t_stcOscFftData;			// length 64 Bytes


// Vorgabewerte CsFastOscillation
typedef   struct
{
	Uint16	usIV;			// Int-Vektor,
	Uint16	usID;			// ID, Untergruppe dieses Vektors fr Tabellen;
	Uint16	usMode;			// Measuring Mode (CS, CR, CSOSC, CDOSC)	
	Uint16	usAmpMode;		// Modulationsfunktion  (Default 1)
	Uint16	usFStrain;		// no longer in use since V-00.01.030
	Uint16	us1;  
	float32	fFreq;
	float32	fAmpl[1];
	float32	fDuration;		// Dauer der Modulation, wenn Mode != 1
	float32	fFStrain;	    // Fast Strain Optimierung Deformation
	float32	fMotorTemp;     // 09.12.15 motor temperature from C-1
	float32 af[8];  		//
}
t_stcFOscSet;	//64-Bytes


#ifdef TODO
// Details zum Motor, wie Kalibrierdaten
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen
	float32 fK1;
	float32 fK2;
	float32 fK3;
	float32 fK4;
	float32 fK5;
	float32 fK6;
	float32 fK7;
	float32 af[8];		//
}
t_stcMotor;			// 64-Bytes

// Details zum Amplifier Motor, wie Kalibrierdaten
typedef   struct
{
	Uint16	usIV;		// Int-Vektor,
	Uint16	usID;		// ID, Untergruppe dieses Vektors fr Tabellen
	float32 fK1;
	float32 fK2;
	float32 fK3;
	float32 fK4;
	float32 fK5;
	float32 fMaxCurrent;
	float32 fResolution;
	float32 af[8];		//
}
t_stcAmplifier;			// 64-Bytes

// Daten fr AC-Amplifier
typedef struct
{
   float32   fCurrent;            // Vorgabewert Strom in A
   float32   fFreq;               // Vorgabewert Frequenz in Hz (10 -400)
   Uint16   usRange;      // Vorgabewert Bereich 0-0,25A (1) oder 0-2,1A (2)
   Uint16   usDirection;   // Vorgabewert Drehrichtung rechts(1) oder links(2)
   Uint16   usCurrentBit;   // Vorgabewert Strom in Bit (0-65535)
   Uint16   usDummy;      // aufffllen durch 4 teilbar, uffbasse Ralfi
}
t_stcAcAmpParameters;
#endif
