/*
 * Firma             :  Thermo HAAKE
 * BU                :  MC
 * Erstelldatum      :  10.07.2001
 * Letzte Aenderung  :  17.07.2001
 *
 * Projekt           :  RS600
 * Verwendungszweck  :  Bereitstellung von Algorithmen zur digitalen 
 *                      Signalverarbeitung
 * Referenzdokumente :  -
 *
 * Autor             :  Uwe Bauer
 * Tester            :  -
 * Teststatus        :  -
 *
 * Editor            :  MS Visual C++ V 6.0
 * Compiler          :  MS Visual C++ V 6.0 / Code Composer V 3.05
 *
 * Versionsnummer    :  1.01
 * Revision History  :  -
 ******************************************************************************/

/*
 * Um mehrfaches Parsen von Quellcode zu vermeiden wird die Header - Datei
 * gekapselt. Alle Deklarationen sollten zwischen den Pre - Prozessor - 
 * Anweisungen stehen.
 */
#ifndef  DSV_H    /* Beginn der Kapselung von DSV_H   */
#define  DSV_H


/*
 * An dieser Stelle des Programmes erfolgt die Deklaration von Makros.
 ******************************************************************************/

#define MEMORY	2


/* Types ********************************************************************/


typedef struct
{
	float32	fMaxEmph;	// maximum Emphases (normaly 1.0)
	float32 fEmph;		// actual Emphases
	float32 fStep;		// step of Emphases
	float32 fOffset;
} t_EmphSig;




/*
 * An dieser Stelle des Programmes erfolgt die Deklaration von 
 * benutzerdefinierten Datentypen.
 ******************************************************************************/

typedef struct
{
   int nActLength;
   int nMaxLength;		
   float32 fData[2][MEMORY];    // Speicher fr die Winkelwerte
   float32 *pfMemoryXnew;       // Zeiger auf aktuellen Wert Zeit
   float32 *pfMemoryYnew;       // Zeiger auf aktuellen Wert Winkel
   float32 *pfMemoryXold;       // Zeiger auf ltesten Wert Zeit
   float32 *pfMemoryYold;       // Zeiger auf ltesten Wert Winkel

} t_DataMemoryStruct;



/* Datenstruktur Lineare Regression nach y=a+bx*/
typedef struct
{
   int   nMaxLength; // maximale Lnge Regressionsbereich
   int   nActLength; // aktuelle Lnge Regressionsbereich
   float32 *pfXi;      // Zeiger auf aktuellen Wert Winkel
   float32 *pfYi;      // Zeiger auf aktuellen Wert Zeit

   float32 fSumX;
   float32 fSumY;
   float32 fSumXx;
   float32 fSumXy;
   float32 fA;         // Gerade Abszissenabschnitt
   float32 fB;         // Gerade Steigung
} t_LinRegressionStruct;






/*
 * Definition des TP-Filters I.Ordnung
 */
typedef struct {
	float32	Fp;			/* Durchlafrequenz [Hz]	*/
	float32	Ts;			/* Abtastzeit [s]			*/
	float32	d0;			/* Zhlerkoeffizient		*/
	float32	c1;			/* Nennerkoeffizient		*/
	float32	uk, uk_1;	/* Eingangsgre			*/
	float32	yk, yk_1;	/* Ausgangsgre			*/
} t_FOLPF;
 

#ifndef MAX_WINDOW_WIDTH
#define	MAX_WINDOW_WIDTH	100
#define	TDL_WIDTH			MAX_WINDOW_WIDTH + 1

#endif
 



/* Regressionsfilter 17.10.06 Kster*/
typedef struct
{
	// todo
   float32 array[10];							//Array
   int   length;								//diskrete Filterlnge
   float32 *pfMemory;   
} t_RegFilterStruct;



/*
 *	Definition des linearen Regressionsfilters
 */
typedef struct LRF_Obj {
	float32 Y1, Y2;							/*	Merkmale											*/
	unsigned short int WindowWidth;	/*	Fensterbreite									*/
	unsigned short int TdlWidth;		/*	max. Anzahl der Vergangenheitswerte		*/
	float32 x[TDL_WIDTH];					/*	unabhaenge Laufvariable						*/
	float32 y1[TDL_WIDTH];				/*	Funktionswerte des ersten Merkmals		*/
	float32 y2[TDL_WIDTH];				/*	Funktionswerte des zweiten Merkmals		*/
	float32 Xmk, Xmk_1;					/*	Mittelwert der unabhaengigen Laufvar.	*/
	float32 Y1mk, Y1mk_1;					/*	Mittelwert des ersten Merkmals			*/
	float32 Y2mk, Y2mk_1;					/*	Mittelwert des zweiten Merkmals			*/
	float32 a1, b1;							/*	Polynomkoeffizienten erstes Merkmal		*/
	float32 a2, b2;							/*	Polynomkoeffizienten zweites Merkmal	*/
	float32 Y1app, Y2app;					/*	Schaetzwerte									*/
	unsigned short int Idx;				/*	allgemeiner Laufindex						*/
} LRF_Obj;

 
/*
 * Definition des Mittelwertbildners mit exponentiell nachlassendem Gedaechtnis
 */ 
typedef struct
{
	float32	uk;			// aktueller Messwert
	float32	yk, yk_1;	// aktueller und alter Mittelwert
	Uint16	M;			// Gedaechtnis
	Uint16	k;			// anwachsendes Gedaechtnis
} t_objEWMAF;

/*
 * Definition des Mittelwertbildners mit anwachsendem Gedaechtnis
 */
typedef struct {
   float	uk;         //	aktueller Messwert
   float	yk;         //	aktueller Mittelwert
   float	yk_1;       //	alter Mittelwert
   float	fK;			// aktuelles Gedchtnis als float
   Uint16	k;          //	aktuelles Gedaechtnis
   Uint16	unMmax;       //	maximales zulaessiges Gedaechtnis
} t_stcAvgIncMem;

/*
 * Definition des Mittelwertbildners mit Ausreisser- / Sprungerkennung
 */
typedef struct {
   Uint16	   S;       //	Grenze zur Unterscheidung zwischen Ausreissern (Z < S)
                     // und Spruengen (Z > S)
   float32	ABW;	   //	Halbe Breite der Toleranzzone
   Uint16     Z;       //	Zaehler fuer aufeinanderfolgende Abweichungen
   float32   uk;      //	aktueller Messwert
   float32   yk;	   //	Ausgang IIR-Mittelwertbildner
   float32   yk_1;	   // alter Wert des IIR-Mittelwertbildners
   Uint16     M;		   //	Gedaechtniszeitkonstante des IIR-Mittelwertbildners
   float32	vk;	   //	Ausgang ANW-Mittelwertbildner
   float32	vk_1;	   //	alter Wert des ANW-Mittelwertbildners
   Uint16	   k;		   //	anwachsendes Gedaechtnis
	} t_stcAvgRS;
	
/*
 * Definition des Mittelwertbildners mit Ausreisser-/Sprungerkennung fuer die
 * gleichzeitige Filterung von zwei Messgroessen.
 */
typedef struct {
	Uint16 S;					// Grenze zur Unterscheidung zwischen Ausreissern (Z<S)
								// und Spruengen (Z>S)
   Uint16     Z;       	//	Zaehler fuer aufeinanderfolgende Abweichungen
	Uint16 M;					// Gedaechtnis des IIR-MWB
	float32 ABW;				// Halbe Breite des Toleranzbands
	float32 u1k;				// aktueller Wert der ersten Messgroesse
	float32 u2k;				// aktueller Wert der zweiten Messgroesse
	float32 y1k, y1k_1;	// IIR-MWB fuer die ersten Messgroesse
	float32 y2k, y2k_1;	// IIR-MWB fuer die zweite Messgroesse
	float32 v1k, v1k_1;	// ANW-MWB fuer die ersten Messgroesse
	float32 v2k, v2k_1;	// ANW-MWB fuer die zweite Messgroesse
	Uint16 k;					// anwachsendes Gedaechtnis
} t_objAvgRS2;


/*
 * Definition des realen Differenzierers mit Tiefpassfilterschaltung
 */
typedef struct {
	float	uk, uk_1;	//	aktuelle und alte Eingangsgroesse z.B. [rad], [K]
	float	vk, vk_1;	// aktuelle und alte Ausgangsgroesse die proportional
								// zur Aenderungsgeschwindigkeit des Eingangssignales
								//	ist z.B. [rad/s], [K/s]
	float	T;				//	Abtastzeit [s]
	float	TD;			//	Differenzierzeit [s]
	float	Td;			//	Daempfungszeit [s]
	float	Dd;			//	Zaehlerkoeffizient der Differenzengleichung [1]
	float	Cd;			//	Nennerkoeffizient der Differenzengleichung [1]
	} t_stcRealDerivativeUnit;

typedef struct {
	float uk, uk_1;	//	aktuelle und alte Eingangsgroesse z.B. [rad], [K]
	float vk;			// aktuelle Ausgangsgroesse die proportional zur
								//	Aenderungsgeschwindigkeit des Eingangssignales
								//	ist z.B. [rad/s], [K/s]
	float T;				//	Abtastzeit [s]
} t_stcIdealDerivativeUnit;

/*
 * Definition der Verzoegerungskette (FIFO) mit den letzten N+1 Messwerten
 */
typedef struct {
	Uint16		N;				//	maximale Laenge
	Uint16		Idx;			//	allgemeiner Laufindex zur Adressierung des naechsten
								//	freien Speicherplatzes
	float32	*pDW;			//	Zeiger auf die erste Speicherstelle
} t_TDL;

/*
 * Definition des gleitenden Mittelwertbildners
 */
typedef struct {
	Uint16		N;		//	Fensterbreite (Gedaechtnis)
	float32	yk;	//	neuer Mittelwert
	float32	yk_1;	//	alter Mittelwert
	t_TDL		sTDL;	//	Verzoegerungskette mit den letzten N+1 - Messwerten
}t_MAF;

/*
 * Definition des Regressionsfilters
 */
typedef struct {
	float32 diff;	//	aktuelle Steigung
	float32 T;		//	Abtastzeit
	float32 *pW;		//	Gewichtsmatrix
	t_TDL	sTDL;		//	Verzoegerungskette mit den letzten n - Messwerten
} t_RGF;


/* Global variables *********************************************************/

/* Functions ****************************************************************/

void InitCondEmphasisSignal(t_EmphSig *pEmph, float32 val, float32 offset);
float32 EmphasisSignal(t_EmphSig *pEmph, float32 val);


/* 
 * Initialisierung der Zustandsgroessen des Mittelwertbildners mit anwachsendem
 * Gedaechtnis
 */
void InitCondAvgIncMem(t_stcAvgIncMem *ps, float value);

/*
 * Vorgabe der maximal zulaessigen Gedaechtnisbreite des Mittelwertbildner mit 
 * anwachsendem Gedaechtnis
 */
Uint16 SetCondAvgIncMem(t_stcAvgIncMem *ps, Uint16 unMemSize);

/*
 * Mittelwertbildner mit anwachsendem Gedaechtnis
 */
float32 AvgIncMem(t_stcAvgIncMem *ps, float32 value);







/***************************************************************************
 * Function:     SetCondLrf
 * Description:  Berechnung der Filtertiefe des Mittelwertbildners
 * 
 * Parameters:    ptr -> Zeiger auf das entsprechende Objekt
 *                p_fValue -> Sollwert einer Gre abhngig derer berechnet
 *                            werden soll
 *                p_nDeep ->  bergabewert Filtertiefe oder "0" fr abh. Berchn.
 * 
 * Return value: 
 * Comments:      5.12.2006 Kster
 ***************************************************************************/
//void SetCondLrf(t_RegFilterStruct *ptr, float32 p_dValue, int p_nDeep);


/***************************************************************************
 * Function:     InitCondLrf
 * Description:  Berechnung der Filtertiefe des Mittelwertbildners
 * 
 * Parameters:    ptr -> Zeiger auf das entsprechende Objekt
 *                p_fValue -> Sollwert einer Gre abhngig derer berechnet
 *                            werden soll
 * 
 * Return value: 
 * Comments:      5.12.2006 Kster
 ***************************************************************************/

//void InitCondLrf(t_RegFilterStruct *ptr, float32 p_dvalue);





/*
   Kurze Beschreibung:
   ==================
      
   Aufruf:
   ======
      LinRegressionFilter(t_filterstruct* filterstruct,int p_nCommand,float p_fActValue)

   Parameter:
   =========
      1.) Adresse des Filterdatenobjekts
      2.) Statusparameter
      3.) aktueller Messwert

   Datum:   19.10.2006
   Autor:   Ralf Kster
*/

//float32 LinRegressionFilter(t_RegFilterStruct* strRegFilter,int p_nCommand,float32 p_fActValue);


float32 FOLPF(t_FOLPF *, float32);
void InitCondFOLPF(t_FOLPF *, float32);
void ParamFOLPF(t_FOLPF *, float32, float32);
 
 
 
//void TDL(LRF_Obj *, float32, float32);
//void InitLRF(LRF_Obj *, unsigned short int);
//void LRF(LRF_Obj *, double, double);



/***************************************************************************
 * Function:     SetCondEWMAF
 * Description:  Berechnung der Filtertiefe des Mittelwertbildners
 * 
 * Parameters:    ptr -> Zeiger auf das entsprechende Objekt
 *                p_fValue -> Sollwert einer Gre abhngig derer berechnet
 *                            werden soll
 *                p_nDeep ->  bergabewert Filtertiefe oder "0" fr abh. Berchn.
 * 
 * Return value: 
 * Comments:      5.12.2006 Kster
 ***************************************************************************/
//void SetCondEWMAF(t_objEWMAF *ptr, float32 p_fValue, int p_nDeep);



/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion werden die Zustandsgroessen des Mittelwertbildners mit
      exponetiell nachlassendem Gedaechtnis mit Startwerten vorbelegt.

   Aufruf:
   ======
      
      InitCondEWMAF(&opjEWMAF, dValue)

   Parameter:
   =========

      1.) Adresse des Filterdatenobjekts
      2.) Startwert (double - Zahlengenauigkeit)

   Datum:   09.01.2004
   Autor:   Uwe Bauer
*/
//void InitCondEWMAF(t_objEWMAF *, float32);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion wird der Mittelwertbildner mit exponetiell nachlassenden
      Gedaechtnis realisiert. Die fuer die Mittelwertbildung erforderliche
      Summe der Messwerte wird in rekursiver Form nachgebildet. Der Algorithmus
      hat die folgende Form:

         yk = yk_1 + (uk - yk_1) / M

   Aufruf:
   ======
      
      EWMAF(&objEWMAF, dValue)

   Parameter:
   =========

      1.) Adresse des Filterdatenobjekts
      2.) aktueller Messwert (double - Zahlengenauigkeit)

   Rueckgabewert:
   =============

      aktueller Mittelwert (double - Zahlengenauigkeit)

   Datum:   09.01.2004
   Autor:   Uwe Bauer
*/
//float32 EWMAF(t_objEWMAF *, float32);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion werden die Zustandsgroessen des Mittelwertbildners mit
      anwachsendem Gedaechtnis mit Startwerten vorbelegt und das bisherige
      Gedaechtnis geloescht.

   Aufruf:
   ======
      
      InitCondAvgIncMem(&tObject, fValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) Startwert (float - Zahlengenauigkeit)

   Datum:   17.07.2002
   Autor:   Uwe Bauer
*/
//void InitCondAvgIncMem(t_stcAvgIncMem *ps, float fValue);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion wird das maximale zulaessige Gedaechtnis des Mittelwert-
      bildners mit anwachsendem Gedaechtnis festgelegt. Liegt die Vorgabe 
      ausserhalb des zulaessigen Bereiches wird die alte Einstellung beibehalten
		und die Statusmedlung NOK.

   Aufruf:
   ======
      
      SetMaxMemSizeAvgIncMem(&tObject, wValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) Gedaechtnis (Datenwort)

   Rueckgabewert:
   =============

      Statusmeldung (Datenwort)

         TRUE  := Gedaechtnis im zulaessigen Bereich und
         FALSE := Gedaechtnis ausserhalb des zulaessigen Bereiches

   Randbedingungen:
   ===============

      Das Gedaechtnis muss im Zahlenbereich eines Datenwortes zu liegen kommen
      und eine Mittelung bewirken (2 <= M <= MAXINTEGER).
      
      siehe auch boolsche Datentypen (Uint16) in typedefs.h

   Datum:   17.07.2002
   Autor:   Uwe Bauer
*/
Uint16 SetMaxMemSizeAvgIncMem(t_stcAvgIncMem *ps, float fValue);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion wird der Mittelwertbildner mit anwachsendem Gedaechtnis
      realisiert. Die fuer die Mittelwertbildung erforderliche Summe der 
      Messwerte wird in rekursiver Form nachgebildet. Der Algorithmus hat die
      folgende Form:

         yk = yk_1 + (uk - yk_1) / k

   Aufruf:
   ======
      
      AvgIncMem(&tObject, fValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) aktueller Messwert (float - Zahlengenauigkeit)

   Rueckgabewert:
   =============

      aktueller Mittelwert (float - Zahlengenauigkeit)

   Datum:   17.07.2002
   Autor:   Uwe Bauer
*/
//float AvgIncMem(t_stcAvgIncMem *ps, float fValue);


/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion werden die Zustandsgroessen des Mittelwertbildners mit
      Startwerten vorbelegt, das bisherige Gedaechtnis geloescht und die 
      Ausreisser- und Sprungerkennungslogik initialisiert.

   Aufruf:
   ======
      
      InitCondAvgRS(&tObject, fValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) Startwert

   Rueckgabewert:
   =============

      ---

   Datum:   16.07.2002
   Autor:   Uwe Bauer
*/
//void InitCondAvgRS(t_stcAvgRS *ps, double fValue);


/*
   Kurze Beschreibung:
   ==================

   Der Filter soll in der Lage sein zwischen Ausreissern und Spruenge zu
   unterscheiden. Aussreisser sollen durch das Filter ausgeblendet, Spruenge
   moeglichst gut wiedergegeben werden. Zur Signalglaettung im stationaeren
   Fall wird ein	IIR-Filter verwendet:

                        yk = yk_1 + (uk - yk_1) / M
   
   Um einem Sprung moeglichst gut folgen zu koennen wird ein ANW-MWB
   eingesetzt:

                        vk = vk_1 + (uk - vk_1) / k

  Die Erkennung von Ausreissern oder Spruengen soll mit Hilfe einer Toleranz-
  zone um den aktuellen gefilterten Wert erfolgen. Liegt ein neuer Messwert
  ausserhalb der Toleranzzone, muss entweder von einem Ausreisser oder einer
  sprungartigen Aenderung ausgegangen werden. Eine Unterscheidung zwischen
  Sprung und Ausreisser ist anhand eines einzigen, ausserhalb der Toleranz
  liegenden Messwertes noch nicht moeglich. Erst wenn mehrere aufeinander-
  folgende Messwerte ausserhalb der Toleranz liegen, kann von einem Sprung
  ausgegangen werden. Er wird, sobald er erkannt wird, an den Ausgang des
  Filters weitergeschaltet. Tauchen die Messwerte kurze Zeit spaeter wieder
  in das Toleranzband ein, hat es sich um einen Ausreisser gehandelt. Er wird
  ausgeblendet. Zur Formulierung dieses Verhaltens als Rechenalgorithmus
  sind die folgenden Kenngroessen definiert:


      S		:=	Grenze zur Unterscheidung zwischen Ausreissern (Z < S) und
               Spruengen (Z > S)
      ABW	:=	Halbe Breite der Toleranzzone
      Z		:=	Zaehler fuer die aufgetretenen Abweichungen
      uk		:= aktueller Messwert
      yk		:=	Ausgang IIR-Mittelwertbildner
      yk_1	:= alter Wert des IIR-Mittelwertbildners
      M		:= Gedaechtnis des IIR-Mittelwertbildners
      vk		:= Ausgang ANW-Mittelwertbildner
      vk_1	:= alter Wert des ANW-Mittelwertbildners
      k		:= Anzahl der aktuellen aufeinanderfolgenden Abweichungen

   (siehe auch Datentyp - Definition t_stcAvgRS)
   
   Aufruf:
   ======
      
      AvgRS(&tObject, fValue)

      oder

      fResult = AvgRS(&tObject, fValue)


   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) aktueller Messwert (double - Zahlengenauigkeit)

   Rueckgabewert:
   =============

      Nutzsignalschaetzwert (double - Zahlengenauigkeit)

   Benutzte Funktionen:
   ===================

      - Betragsfunktion fabs aus der mathematischen Bibliothek (math.h)

 Datum:   16.07.2002
 Autor:   Uwe Bauer
*/
//double AvgRS(t_stcAvgRS *ps, double fValue);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion wird die halbe Breite der Toleranzzone des
      Mittelwertbildners mit Ausreisser-/Sprungerkennung vorgegeben.

   Aufruf:
   ======
      
      SetTolAvgRS(&tObject, fValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) halbe Breite der Toleranzzone

   Rueckgabewert:
   =============

      Statusmeldung

         TRUE  := neue Toleranzbandbreite ubernommen
         FALSE := alten Wert der Toleranzbandbreite beibehalten

   Randbedingungen:
   ===============

      Die Vorgabe muss betragsmaessig erfolgen (ABW > 0).

   Datum:   16.07.2002
   Autor:   Uwe Bauer
*/
//Uint16 SetTolAvgRS(t_stcAvgRS *ps, double fValue);

/*
   Kurze Beschreibung:
   ==================

      Mit der Funktion wird das Gedaechtnis M

            yk = yk_1 + (uk - yk_1) / M

      des IIR - Mittelwertbildners vorgegeben.

   Aufruf:
   ======
      
      SetMemAvgRS(&tObject, wValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) Gedaechtnis

   Rueckgabewert:
   =============

      Statusmeldung der Parameteruebernahme (TRUE/FALSE)

   Randbedingungen:
   ===============

      Betrachtet man den rekursiven Algorithmus

         yk = yk_1 + (uk - yk_1) / M

      so erkennt man, dass erst fuer ein Gedaechtnis der Breite M = 2 eine
      Mittelung erfolgt.

   Datum:   16.07.2002
   Autor:   Uwe Bauer
*/
//Uint16 SetMemAvgRS(t_stcAvgRS *ps, Uint16 wValue);

/*
   Kurze Beschreibung:
   ==================
      Mit der Funktion wird die Grenze S zur Unterscheidung zwischen
      Ausreissern und Spruengen vorgegeben.

   Aufruf:
   ======
      
      SetCntLimAvgRS(&tObject, wValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Filterdatenstruktur
      2.) Grenzwert

   Rueckgabewert:
   =============

      Statusmeldung der Parameteruebernahme (TRUE/FALSE)

   Randbedingungen:
   ===============

      Ein Ausreisser ist mindestens ein Messwert der ausserhalb der Toleranz
      liegt. Somit muss zur Unterscheidung zwischen Ausreissern und Spruengen
      die Grenze (S > 1) gewaehlt werden.

   Datum:   16.07.2002
   Autor:   Uwe Bauer
*/
//Uint16 SetCntLimAvgRS(t_stcAvgRS *ps, Uint16 wValue);

/*
   Kurze Beschreibung:
   ==================
      Mit der Funktion wird die Aenderungsgeschwindigkeit in einem gemessenen
		Signal bestimmt. Dazu wird der reale Differenzierer verwendet. Zur 
		Formulierung dieses Verhaltens als Rechenalgorithmus sind die folgenden
		Kenngroessen definiert:

			TD		:=	Differenzierzeit
			Td		:= Daempfungszeit
			T		:= Abtastzeit

		Durch die zusaetzliche Verzoegerungsschaltung im Nenner die als Tiefpass-
		filter 1.Ordnung aufgefat werden kann ist die Bedingung Zaehlerordnung
		kleiner gleich Nennerordnung erfuellt. Ansonsten waere die Sprungantwort
		des Differenzierers die technisch nicht realisierbare Impulsantwort. Ein
		weiteres Argument fuer die Verzoegerungsschaltung im Nenner liegt darin
		begruendet, da im gefilterten Mesignal noch immer hochfrequente 
		Stoerungen enthalten sein koennen. Bei der Gewinnung der Nutzinformation
		durch Differentiation eines solchen Signals ist die gewonnen Nutz-
		information stark verrauscht. Normalerweise ist die Frequenz der Stoerung
		sehr viel groesser als eins. Nach der Differentiation taucht diese groe
		Stoerfrequenz in der Amplitude der Nutzinformation auf und verfaelscht
		diese. Durch vorherige Tiefpassfilterung wird durch geeignete 
		Dimensionierung der Daempfungszeitkonstante dieser Effekt stark minimiert.
		Die Differenzengleichung  des realen Differenzierers lautet:

		   vk = Dd * (uk - uk_1) + Cd * vk_1

   Aufruf:
   ======
      
      RealDerivativeUnit(&tObject, fValue)

   Parameter:
   =========

      1.) Adresse der betroffenen Differenziererdatenstruktur
      2.) gemessenes Signal

   Rueckgabewert:
   =============

      Aenderungsgeschwindigkeit des gemessenen Signales

   Datum:   29.08.2002
   Autor:   Uwe Bauer
*/
//double RealDerivativeUnit(t_stcRealDerivativeUnit *ps, double fValue);

/*
   Kurze Beschreibung:
   ==================
      Mit der Funktion werden die Koeffizienten der Differenzengleichung des 
		realen Differenzierers berechnet.

   Aufruf:
   ======
      
      RDUDiffEquaCoeff(&tObject, fValue1, fValue2, fValue3)

   Parameter:
   =========

      1.) Adresse der betroffenen Differenziererdatenstruktur
      2.) Differenzierzeit
		3.) Daempfungszeit
		4.) Abtastzeit

   Rueckgabewert:
   =============

      Statusmeldung der Berechnung (OK/NOK)

   Randbedingungen:
   ===============

		Saemtliche Zeiten muessen groesse als Null sein. Zusaetzlich wird die
		parktisch realisierbare Abtastzeit durch die Hardware begrenzt.

   Datum:   29.08.2002
   Autor:   Uwe Bauer
*/
/*Uint16 DiffEquaCoeffRDU(t_stcRealDerivativeUnit *ps,
								  double	TD, double Td, double T);
*/
/*
   Kurze Beschreibung:
   ==================
      Mit der Funktion werden die Zustandsgroessen des realen Differenzierers
		mit Startwerten vorbelegt.

   Aufruf:
   ======
      
      InitCondRDU(&tObject, fValue1, fValue2)

   Parameter:
   =========

      1.) Adresse der betroffenen Differenziererdatenstruktur
      2.) Startwert der Eingangsgroesse (Messwert)
		3.) Startwert der Ausgangsgroesse (Aenderungsgeschwindigkeit)
		4.) Abtastzeit

   Datum:   29.08.2002
   Autor:   Uwe Bauer
*/
/*
void InitCondRDU(t_stcRealDerivativeUnit *ps, double Uo, double Vo);
float IdealDerivativeUnit(t_stcIdealDerivativeUnit *ptr, float fValue);
Uint16 SetCondIDU(t_stcIdealDerivativeUnit *ptr, double T);
void InitCondIdealDerivateUnit(t_stcIdealDerivativeUnit *ptr, float fValue);
*/

//Uint16 CreateTDL(t_TDL *ptr, float length);
//void DeleteTDL(t_TDL *ptr);
//void InitTDL(t_TDL *ptr, float32);
//void TappedDelayLine(t_TDL *ptr, float32 value);
//void InitCondTDL(t_TDL *ptr, float32 value);

//Uint16 CreateMAF(t_MAF *ptr, float, double);
/*
Uint16 CreateMAF(t_MAF *ptr, float);
void DeleteMAF(t_MAF *ptr);
void InitCondMAF(t_MAF *ptr, double value);
double MovingAverageFilter(t_MAF *ptr, double value);
Uint16 CreateRGF(t_RGF *ptr, float length, double sampletime);
Uint16 CreateWeightMatrixRGF(t_RGF *ptr, float length, double sampletime);
void DeleteRGF(t_RGF *ptr);
void InitCondRGF(t_RGF *ptr, double value);
double RegressionFilter(t_RGF *ptr, double value);
*/
//void AvgRS2(t_objAvgRS2 *ps, double fValue1, double fValue2);


//void SetCondXYDataMemory(t_DataMemoryStruct *p_pMemory, int p_nLength);
//void ResetCondXYDataMemory(t_DataMemoryStruct *p_pMemory, double p_fX);
//void StoreXYData(t_DataMemoryStruct *p_pMemory, float32 fX, float32 fY);

//float32 LinRegressionXY(t_LinRegressionStruct *ptLrd, t_DataMemoryStruct *ptXYData, int nLength);
//float32 LinearRegressionXY(t_LinRegressionStruct *ptLrd, t_DataMemoryStruct *ptXYData, int nLength);

#endif   /* Ende der Kapselung von DSV_H */
