/**
 * @file    filt.h
 *
 *          Digital filter functions
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsrue, Germany
 *
 *          (c) Copyright 2010, Thermo Electron
 *
 *          All rights reserved. Thermo Electron source code is an unpublished work and the use of a copyright notice
 *          does not imply otherwise. This source code contains confidential, trade secret material of Thermo Electron.
 *          Any attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source
 *          code is strictly prohibited, unless the prior written consent of Thermo Electron is obtained.
 *
 * @par     Creation Date:
 *
 *          15-Feb-2010
 *
 * @par     Last Modification:
 *
 *          06-Apr-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 *
 *          1.00.001    Initial version
 */

#ifndef FILT_H
#define FILT_H                                                  /**< File encapsulation to avoid multiple parsing.                */

// data buffer length for Moving Average Filter Objects for 1/4 turn
#define SIG_MAVG_LENGTH     250 // ab Version 01.06.155 wieder 250 statt 125 Werte
//
#define FILTER_FULL ((Uint16) 1)

/*
 **********************************************************************************************************************************
 *                                                            INCLUDES                                                            *
 **********************************************************************************************************************************
 */

//#include "typedefs.h"

/*
 **********************************************************************************************************************************
 *                                                       GLOBAL DATA TYPES                                                        *
 **********************************************************************************************************************************
 */
typedef struct filt_mavg_obj {
    Uint16      uN;             // Buffer size
    float32     *pfData;        // data-buffer start adress
    Uint16      uK;             // Number of samples in the buffer
    float32     fSum;           // 
    float32     fY;             // Moving average value
    Uint16      fFull;          // Buffer load status
} objMavg;

// Struktur Daten fr Moving Average Torque
typedef struct
{
    Uint16	unLen;                      // number of data for Moving Average Filter
    Uint16	unState;                    // state of Filter 0 - in progress, 1 - Ready
    Uint16  unIdx;                      // Index calculated from Phi-absolute
    Uint16  unIdxOld;                   // Index from last sample
    float32 *pfDatBuf;                  // DataBuffer with N elements
    //float32 afDatBuf[SIG_MAVG_LENGTH];   
    objMavg objSigMavg;                 // objct for Torque moving average filter
} t_stcFiltMavg;





/*
 **********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************
 */


float32 FilterMAvg (objMavg *p, const float32 p_fData);

float32 FiltMAvg(objMavg *p, const float32 u);

/**
 * Initialize a moving average filter
 *
 * @param   p       Pointer to the used filter instance.
 * @param   data    Pointer to the used data memory.
 * @param   n       Window length.
 */
void FiltMAvgInit(objMavg *p, float32 *data, const Uint16 n);

//
void InitSigMavg(t_stcFiltMavg *p_stcSig);
//
void InitCondSigMavg(t_stcFiltMavg *p_stcSig);
//
void ResetSigMavg(t_stcFiltMavg *p_stcSig);
//
float32 SignalCalcMavg(t_stcFiltMavg *p_stcSig, float32 p_fActSig, float32 p_fPhiAbs);
//
void FiltLinRegInit(objMavg *p, float32 *data, const Uint16 n);
// linear Regression over a defined data array x-y
float32 FilterLinReg (objMavg *p, float32 p_fDataX, float32 p_fDataY);


#endif /* FILT_H    */
