/*
*/
#ifndef _global_h
#define _global_h

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ********** select device variant *******************************************
// VTiQ with Air-bearing, if not comment out
#define VTIQ_AIR
// ********** select PWM Frequency ********************************************
#define EPWMFRQ_25KHZ
//#define EPWMFRQ_50KHZ

//#define TEST_DEBUG

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// low current version with 0,120 Ohm resistances
//#define NANO

// Todo, already in use, but poss. to eliminate
//#define DSP_TARGET F28335

#if !defined(TRUE)
#define  TRUE ((Uint16) 1)                   /* wahr                          */
#endif

#if !defined(FALSE)
#define  FALSE ((Uint16) 0)                  /* falsch                        */
#endif

#if !defined(OK)
#define  OK	((Uint16) 1)                  /* Zustand in Ordnung            */
#endif

#if !defined(NOK)
#define  NOK   ((Uint16) 0)                  /* Zustand nicht in Ordnung      */
#endif

#if !defined(ON)
#define  ON    ((Uint16) 1)                  /* einschalten                   */
#endif

#if !defined(OFF)
#define  OFF   ((Uint16) 0)                  /* ausschalten                   */
#endif

#if !defined(MIN)
#define MIN(a, b) (((a) < (b)) ? (a) : (b))  /* Minimum zweier Zahlen         */
#endif

#if !defined(MAX)
#define MAX(a, b) (((a) > (b)) ? (a) : (b))  /* Maximum zweier Zahlen         */
#endif


#define	CLEAR	0
#define	SET		1
#define	PROOF	2
#define	SEND	3


// Abtastzeiten/Sampling time in s
#define ST_200US 200
#define ST_1000US 1000
#define ST_2000US 2000
#define ST_2500US 2500

#define TIME_20US	0.00002
#define TIME_2MS	0.002

#define TIME_20MS	0.02
#define TIME_60MS	0.06
#define TIME_100MS	0.1
#define TIME_200MS	0.2
#define TIME_250MS	0.25
#define TIME_500MS	0.5
#define TIME_750MS	0.75
#define TIME_1000MS	1.0
#define TIME_1500MS	1.5
#define TIME_2000MS	2.0
#define TIME_3000MS	3.0
#define TIME_5000MS	5.0


// Frequencies for FOLP Filter
#define F25KHZ	((float32)25000.0)
#define F1000HZ	((float32)1000.0)
#define F50HZ	((float32)50.0)
#define F250HZ	((float32)250.0)
#define F10HZ	((float32)10.0)
#define F1HZ	((float32)1.0)


#define CPU_RATE    6.667L   // for a 150MHz CPU clock speed (SYSCLKOUT)

/*!	\def CPU_FRQ
 * 	\brief CPU (SYSCLK) in HZ	*/
#define	CPU_FRQ								(150e6)		// 150 MHz
#define	CPU_CLOCK							(1/CPU_FRQ)

/*!	\def LOSP_FRQ
 * 	\brief Low Speed Clock in HZ	*/
#define	LOSP_FRQ							(75e6)

/*!	\def EPWM_FRQ
 * 	\brief Frequenz der PWM-Ausgangssignale in HZ	*/
// for test use to reduce CMRR of AD8210 only 25 kHz
#ifdef EPWMFRQ_25KHZ
    #define	EPWM_FRQ						(25e3)		// 25 kHz
#else
    #define	EPWM_FRQ						(50e3)		// 50 kHz
#endif

#define	EPWM_CLOCK							(1/EPWM_FRQ)
#define EPWM_PERIOD_US						(EPWM_CLOCK*1e06)

/*!	\def MEP_STEP_SIZE
 * 	\brief Micro Edge Positioning (MEP) step size, 150ps	*/
#define MEP_STEP_SIZE						(150e-12)		// Micro Edge Positioning (MEP) step size, 150ps

/*!	\def PWM_ERROR_MAX_COUNT
 * 	\brief Maximale Anzahl von berstromabschaltungen des PWM-Moduls innerhalb
 * 	1ms, welches zur kompletten Abschaltung fhrt, EDC: 50, TFS: 50	*/
#define PWM_ERROR_MAX_COUNT					50

/*!	\def PWM_TOTZEIT_NS
 * 	\brief Angabe der Totzeit (DEADTIME) in ns, was 70ns from EDC, changed to 120ns bo. power lost! */
#define PWM_TOTZEIT70_NS						70
#define PWM_TOTZEIT120_NS						120

#define	FOC_BERECHNUNG_ALLE_X_MAL			1


// DO NOT MODIFY THIS LINE.
//#define DELAY_US(A)  DSP28x_usDelay(((((long double) A * 1000.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)
//#define DELAY_NS(A)  DSP28x_usDelay(((((long double) A * 1.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)

extern void DSP28x_usDelay(Uint32 Count);

#ifdef RAUS
typedef enum{
	NoError = 0,//!< Kein Fehler ist aufgetreten
	AdsRecvConfig,//!< Die gesendete Konfiguration stimmt mit der zurckgelesenen nicht berein
	Ip2000RecvID,//!< Die empfangene ID ist nicht 0b1001 fr GC-IP2000
	Ip2000RecvEepTimeout,//!< Beim Lesen des Eepromstatusregisters ist das Bit BSY nicht in der gewnschten Zeit auf 0 gesetzt worden
	Ip2000RecvEepExist,//!< Es ist kein Eeprom angeschlossen, der Ip2000 aber behauptet es wre einer angeschlossen
	Ip2000RecvCFG1,//!< Die gesendete Konfig fr CFG1 stimmt nicht mit der Empfangenen berein
	Ip2000RecvCFG2,//!< Die gesendete Konfig fr CFG2 stimmt nicht mit der Empfangenen berein
	Ip2000RecvRSV,//!< Die gesendete Konfig fr RSV stimmt nicht mit der Empfangenen berein
	Ip2000RecvError//!< Der empfangene Triggerwert enthlt ein gesetztes Errorbit
} ErrorCodes;
#endif



#endif
