/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         typedef.h
* @brief        Portable data types
* @details      The direct use of standard C data types must be avoided because their size is not portable. Instead, the data types
*               in this file should be used. Depending on the target processor and compiler the data types must be adapted.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef TYPEDEF_H
#define TYPEDEF_H

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                                        /* See Note #1.                               */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/


//---------------------------------------------------------------------------
// For Portability, User Is Recommended To Use Following Data Type Size
// Definitions For 16-bit and 32-Bit Signed/Unsigned Integers:
//

typedef int             int16;
typedef signed long int int32;
typedef unsigned int    Uint16;
typedef unsigned long   Uint32;
typedef float           float32;


#if !defined(CHAR)
typedef char CHAR;																	/**< 8-bit character                          */
#endif

#if !defined(BOOLEAN)
typedef unsigned char BOOLEAN;														/**< 8-bit boolean or logical                 */
#endif

#if !defined(INT08U)
typedef unsigned char INT08U;														/**< 8-bit unsigned integer                   */
#endif

#if !defined(INT08S)
typedef signed char INT08S;															/**< 8-bit signed integer                     */
#endif

#if !defined(INT16U)
typedef unsigned int INT16U;														/**< 16-bit unsigned integer                  */
#endif

#if !defined(INT16S)
typedef signed int INT16S;															/**< 16-bit signed integer                    */
#endif

#if !defined(INT32U)
typedef unsigned long int INT32U;													/**< 32-bit unsigned integer                  */
#endif

#if !defined(INT32S)
typedef signed long int INT32S;														/**< 32-bit signed integer                    */
#endif

#if !defined(INT64U)
typedef unsigned long long INT64U;													/**< 64-bit unsigned integer                  */
#endif

#if !defined(INT64S)
typedef signed long long INT64S;													/**< 64-bit signed integer                    */
#endif

#if !defined(FP32)
typedef	float  FP32;																/**< 32-bit floating point                    */
#endif

#if !defined(FP64)
typedef	float FP64;															/**< 64-bit floating point                    */
//typedef	long double FP64;															/**< 64-bit floating point                    */
#endif

#if !defined(REG08)
typedef volatile INT08U REG08;                                                      /**< 8-bit register                           */
#endif

#if !defined(REG16)
typedef volatile INT16U REG16;                                                      /**< 16-bit register                          */
#endif

#if !defined(REG32)
typedef volatile INT32U REG32;                                                      /**< 32-bit register                          */
#endif

#if !defined(REG64)
typedef volatile INT64U REG64;                                                      /**< 64-bit register                          */
#endif

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                       FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                                              /* End of TYPEDEF.H file.                     */
