/*!
\file	Ccmode.c
\brief	here we find the functions for the CC-Mode
		CC means Constant Current control of the EC-Motor,
		this is not astandard, but more a test mode. The motor
		remains in CC-Mode when e.g. no measurement has been startet.

\author	Koester
\date	19.10.2011
\details
		changes:   12.09.16, additional channels for test functions
		
*/

// for Compiler control
#define CCMODE

/* Include files **************************************************************/
#include "Project.h"
#include "mth.h"
#include "Uc1dsp.h"
#include "Measmode.h"
#include "Ccmode.h"
#include "timer.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "correction.h"
#include "angle.h"
#include "math.h"
#include "Device.h"
#include "Test.h"
#include "control.h"
#include "Pmsm.h"

/* Macro definitions **********************************************************/


/* Types **********************************************************************/
// Struktur Daten nur fr CC-Mode
typedef struct
{
	//Uint16	usMode;		// Modus not used at the moment
	float32 fIout;
} t_stcCcMode;



/* Global variables *********************************************************/

// Datastrcture for CC-Mode, must not be pre-initilized
t_stcCcMode tCcMode;


/* Functions ****************************************************************/

/*!
\fn		void InitCsMode(void)
\brief  CR-Mode initialisieren
\author	Kster
\date	25.10.2011

\param[in]
\return	void

\details
	changes:
*/
void CcModeInit(void)
{
	// nothing to do at the moment
	asm(" nop");
}



/*!
\fn		CcMode(float32 p_fSollwertw)
\brief  Vorgabe Motorstrom
\author	Kster
\date	05.01.2012
\param[in]
\return	void

Changes:

*/  

void CcMode(float32 p_fSollwertw)
{
	/* Umspeichern Sollwert  */
	tCcMode.fIout = p_fSollwertw;
	// Anzeige-, Messwert Drehmoment speichern
	*PtrTorqueMeasValue() = 0.000;
	// Anzeige-, Messwert Omega speichern
	*PtrOmegaMeasValue() = *PtrOmegaAvg();
	// 
	*PtrPhiMeasValue() = *PtrAngleRel();
	// Anzeige-, Messwert Absolutwinkel speichern
	*PtrPhiAbsMeasValue() = *PtrAngleAbs();
	// Store actual Omega for e.G. Mode changes
	*PtrOmegaOld() =*PtrOmegaAvg();
#ifdef TEST_DEBUG
  // 12.09.16, additional channels for test functions
  *PtrMeasTestF6() = *PtrFocTestI1();
  *PtrMeasTestF7() = *PtrFocTestI2();
  *PtrMeasTestF8() = *PtrFocTestI3();
#endif
	// Kalibrierung: Strom[Bit]=f(Drehmoment) [Nm]
	*PtrFocSetCurrent() = tCcMode.fIout;
}
