/*!
\addtogroup Device
\file	Device.c
\brief	Device and Sensor Constants Control.
		
		
\author	Koester
\date	03.01.2012
\details
	changes:	01.10.12 Standardising all Device-, Sensor- and Motorconstants and
						and the transfer from Master-C, rename variables.	

*/

//#define DEVICE	/* zur Compilersteuerung */

#include "Project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
#include "device.h"
#include "Uc1dsp.h"
#include "SsiComm.h"
#include "Measmode.h"
#include "Csmode.h"
#include "timer.h"
#include "correction.h"
#include "math.h"

/* Macro definitions ********************************************************/

// effective value maximum motor current (uncalibrated)
#define IMAX_MOTOR  (float)3.68284782
// resolution motor current
#define I_Motor_RES (float)32768.0
// one bit of motor current (resolution 32768, max. 5,20833333 A)
#define I_MOTOR_BIT (float)158.94572e-06


// standard value motor constant [Nm/A]
#define K_Motor (float)0.0301
// standard value torque rsolution (uncalibrated)
#define TRQ_RESOLUTION (IMAX_MOTOR * K_Motor) / I_Motor_RES

/* Types ********************************************************************/

/* Global variables *********************************************************/

// Details zum Geraet
t_stcDev tDev;          // Details zum Geraet

// Details zum aktuellen Sensor. A-, M-Faktor....
t_stcSensor tSensor;


/* Functions ****************************************************************/

// *** Device Data for Motor *******************************

float* PtrPolAngle(void)
{
	return &tDev.afC[DC18];
}



/*!
\fn		float* PtrCal1Motor(void)
\brief  Reading or writing Calibration constant Motor. 
		Write with:	*PtrCal1Motor() = float;
		Read witht:		= *PtrCal1Motor(); 

\author	Kster
\date	01.10.2012
\param[in]
\return	void
\details Changes:

*/  
float* PtrCal1Motor(void)
{
    return &tDev.fKmotor;
}

// dito, torque resolution
float* PtrImResolution(void)
{
    return &tDev.fCurrRes;
}


// dito, Kp from CR controller
float* PtrCrKP(void)
{
	return &tDev.afC[DC8];
}

// dito, d0i from CR controller
float* PtrCrD0(void)
{
	return &tDev.afC[DC9];
}



/*!
\fn		Uint16* PtrDevicetype(void)
\brief  Lesen oder speichern Gertetyp. 
\author	Kster
\date	01.10.2012
\param[in]
\return	void
Changes:
*/  
float* PtrDevicetype(void)
{
	return &tDev.afC[TYPE];
}


/*!
\fn		float* PtrTorqueMax(void)
\brief  Lesen oder speichern Messwert Drehmoment. 
		Speichern mit:	*PtrTorqueMax() = float;
		Lesen mit:		= *PtrTorqueMax(); 

\author	Kster
\date	01.10.2012
\param[in]
\return	void
\details Changes:

*/  
float* PtrTorqueMax(void)
{
	return &tDev.afC[TORQUMAX];
}



/*!
\fn		float* PtrOmegaMax(void)
\brief  Lesen oder speichern Messwert Drehmoment. 
		Speichern mit:	*PtrOmegaMax() = float;
		Lesen mit:		= *PtrOmegaMax(); 

\author	Kster
\date	01.10.2012
\param[in]
\return	void
\details Changes:

*/  
float* PtrOmegaMax(void)
{
	return &tDev.afC[OMEGAMAX];
}

// dito Rawdata Semaphor
float* PtrRawdataFlag(void)
{
	return &tDev.afC[RAWDATA];
}


// dito Inertia Sensor
float* PtrInertiaDevice(void)
{
	return &tDev.afC[J_MOTOR];
}

// dito Inertia Sensor
float* PtrInertiaSensor(void)
{
	return &tSensor.afC[J_SENS];
}

// dito Sensor Constant A-Factor
float* PtrSensorA(void)
{
	return &tSensor.afC[AFACTOR];
}

// dito Sensor Constant M-Factor
float* PtrSensorM(void)
{
	return &tSensor.afC[MFACTOR];
}

// dito Sensor Constant Compliance
float* PtrSensorCompliance(void)
{
	return &tSensor.afC[COMPLIANCE];
}


// dito Encoder Constant
float* PtrEncoderConst(void)
{
	return &tDev.fEncoderConst;
}


// *** Device Data for Electronic **************************


/*!
\fn		float* PtrCal1Motor(void)
\brief  Reading or writing Calibration constantc Electronic.
		Here: Offset and Calibration-Factor for channel-1, -2, -3 
		Write with:	*PtrIoff1Ampl() = float;
		Read witht:		= *PtrIoff1Ampl(); 

\author	Kster
\date	07.06.2013
\param[in]
\return	void
\details Changes:

*/  
float* PtrIoff1Ampl(void)
{
	return &tDev.afC[IOFF_1_AMPL];
}

float* PtrIoff2Ampl(void)
{
	return &tDev.afC[IOFF_2_AMPL];
}

float* PtrIoff3Ampl(void)
{
	return &tDev.afC[IOFF_3_AMPL];
}

float* PtrIcal1Ampl(void)
{
	return &tDev.afC[ICAL_1_AMPL];
}

float* PtrIcal2Ampl(void)
{
	return &tDev.afC[ICAL_2_AMPL];
}

float* PtrIcal3Ampl(void)
{
	return &tDev.afC[ICAL_3_AMPL];
}





/*!
\fn		void DeviceSensorInit(void)
\brief  Initializing Device constants for proper start!

\author	Kster
\date	24.10.2011
\param[in]
\return	
\details
	Changes:

*/  
void DeviceSensorInit(void)
{
	// calculate Encoder constant from number of counts and interpolation rate
	tDev.fEncoderConst = (float) (CONST_ENCODER * CONST_INTERPOL);
	// Meas.-System initializing
	tDev.afC[TYPE] = DEV_VTIQ;
	tDev.afC[CAL1_MOTOR] = KM_MOTOR;

	// todo Vorbelegung bei RAM Version, 10.04.15 auch bei FLASH, dicker Bug
	tDev.afC[CAL2_MOTOR] = 0.0;
	tDev.afC[CAL3_MOTOR] = 0.0;
	tDev.afC[CAL4_MOTOR] = 0.0;
	tDev.afC[J_MOTOR] = 33.0e-06;	
	tDev.afC[TORQUMAX] = 0.102;
	//tDev.afC[OMEGAMAX] = 209.43951;	// 2000 min-1
	// 06.09.2016 testweise wegen Pruefung High-Shear
	tDev.afC[OMEGAMAX] = 324.63124;	// 2000 min-1

	tDev.afC[IMAX_AMPL] = IMAX_MOTOR;	// A eff.
	tDev.afC[IOFF_1_AMPL] = 0.0;
	tDev.afC[IOFF_2_AMPL] = 0.0;
	tDev.afC[IOFF_3_AMPL] = 0.0;
	tDev.afC[ICAL_1_AMPL] = I_MOTOR_BIT;
	tDev.afC[ICAL_2_AMPL] = I_MOTOR_BIT;
	tDev.afC[ICAL_3_AMPL] = I_MOTOR_BIT;
	// Sensor-System initializing for CC24
	tSensor.afC[TYPE] = STYP_ZYL;
	tSensor.afC[AFACTOR] = 1.0;
	tSensor.afC[MFACTOR] = 1.0;
	tSensor.afC[DAMP] = 0.0;
	tSensor.afC[J_SENS] = 0.0e-06;
	tSensor.afC[COMPLIANCE] = 0.0057;
    // =01.02.16 IMPORTANT TO IMPROVE TORQUE RESOLTUION
    tDev.fCurrRes = I_MOTOR_BIT ;
}
