/*!
\file	Diagnos.c
\brief	functions for updating diagnose data

\author	Koester
\date	01.10.2012
\details
	changes: 14.01.2014	Diagnose for PWM integrated in MotorControl diagnose


*/


#define DIAGNOS

/* Include files ************************************************************/
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "Uc1dsp.h"
#include "Diagnos.h"
#include "SpiA.h"
#include "Epwm.h"
#include "GCIP2000.h"


/* Macros *******************************************************************/

/* Types ********************************************************************/

/* Local Prototypes *********************************************************/

/* Global variables *********************************************************/

t_stcDiagnoseData tDiagnose;

/* Functions ****************************************************************/


/*!
\fn		Uint16* PtrDsCpu(void)
\brief  Read or Write actual CPU state 
		Write with:	*PtrDsCpu() = Uint16;
		Read with:	= *PtrDsCpu(); 

\author	Kster
\date	02.01.2012
\param[in]
\return	void

\details Changes:

*/  
Uint16* PtrDsCpu(void)
{
	return &tDiagnose.ausDs[CPU];
}


//dito actual Diagnosis-State of Motorcontrol
Uint16* PtrDsMotorCon(void)
{
	return &tDiagnose.ausDs[MOTORCON];
}


//dito actual Diagnosis-State of PWM Modul
Uint16* PtrDsPwm(void)
{
	return &tDiagnose.ausDs[PWM];
}


//dito actual Diagnosis-State of Interpolation Encodersignals
Uint16* PtrDsIP(void)
{
	return &tDiagnose.ausDs[IP];
}


//dito actual Diagnosis-State of communication via SSI
Uint16* PtrDsComm(void)
{
	return &tDiagnose.ausDs[COMM];
}

Uint16* PtrDsMemory(void)
{
	return &tDiagnose.ausDs[MEMALLOC];
}



/*!
\fn		void InitDiagnosStates(void)
\brief	Initialize all Diagnos-states at program start

*/
void InitDiagnosStates(void)
{
	tDiagnose.ausDs[CPU] = 0;
	tDiagnose.ausDs[IP] = 0;
	tDiagnose.ausDs[MOTORCON] = 0;
	tDiagnose.ausDs[PWM] = 0;
	tDiagnose.ausDs[COMM] = 0;
    tDiagnose.ausDs[MEMALLOC] = 0;
}


unsigned int unError;
#ifdef RAUS
void DiagnosCheckStates(void)
{
	// todo 29.07.13 check for Errors in IP2000 state, no action at the moment
	if( (*PtrDsIP() & 0x00FF) != IP_NOERROR)
	{
		SpiADeinitInterruptSendRecv();
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		//PWMDisable();
		//Fehler auslesen und Counter von IP reseten
		unError = SpiARead16(GCIP_ADR_STAT);
		//SpiAWrite8Buffer(GCIP_CMD_RESCNT, GCIP_ADR_CMD);
		asm(" nop");
		// todo
		//ESTOP0;
		// !!!! Zum wieder Starten !!!!
		// SpiAInitInterruptSendRecv();
		// PWMEnablr();
	}
	//if( (*PtrDsMotorCon() & 0x000F) != MC_NOERROR )
	//{
		//PWMDisable();
		//NOP for Breakpoint
//		asm(" nop");
		
//		!!!! Zum wieder Starten !!!!
//		SpiAInitInterruptSendRecv();
//		PWMEnablr();
//	}

}
#endif


#ifdef RAUS
Uint16 ChecksumDataSSI(void *p_pData)
{
	Uint16 unSize;
	Uint16 unChecksum;
	char *pcData;
	// Char-Pointer auf Adresse Struct
	pcData = (char*)p_pData;
	unChecksum = 0;
	// Anzahl WORDs ermitteln
	unSize = SSIBUFSIZE;
	//unSize = sizeof(*pcData);
	// Summe Daten mit 16-Bit Addition ermitteln
	while(unSize > 0)
	{
		unChecksum = unChecksum + *pcData;
		unSize = unSize - 1;
		pcData++;
	}
	return(unChecksum);
}




Uint16 ChecksumDataSSI(void *p_pData)
{
	Uint16 unSize;
	Uint32 unChecksum;
	Uint32 *pcData;
	// Char-Pointer auf Adresse Struct
	pcData = (Uint32*)p_pData;
	unChecksum = 0;
	// Anzahl WORDs ermitteln
	unSize = SSIBUFSIZE/2;
	//unSize = sizeof(*pcData);
	// Summe Daten mit 16-Bit Addition ermitteln
	while(unSize > 0)
	{
		unChecksum = unChecksum + *pcData;
		unSize = unSize - 1;
		pcData++;
	}
	return(unChecksum);
}
#endif

