/*!
\addtogroup group_name_api
\file	Epwm.c
\brief	Document:	TMS320x2833x, 2823x Enhanced Pulse Width Modulator (ePWM) Module SPRUG04A
		This modul contains all EPWM-Modul functions for the High-Resolution-EPWM-Modul.
		with the Event-Trigger-Submodul of EPWM2 and-3 the ADS8329 and the IP2000 are triggered
		to start a conversion.

\author	Schneider, Koester
\date	31.05.2012
\details
		

*/

#include "Project.h"		// all global things as Type definitions, Hardware def., ...
//#include "DSP2833x_EPwm_defines.h"
#include "Epwm.h"
#include "Uc1dsp.h"
#include "Diagnos.h"
#include "Measmode.h"
//#include "DSP2833x_Device.h"     // DSP2833x Headerfile Include File
//#include "DSP2833x_GlobalPrototypes.h"
//#include "hardware.h"
//#include "IQmathLib.h"
//#include "Pmsm.h"
//#include "ADS8329.h"


/* Macro definitions ********************************************************/


/* Types ********************************************************************/
// Data only for Epwm Modul
typedef struct
{
	Uint16	usErrorCnt ;	// number of overcurrent errors at EPWM1-TZINT
	Uint16	usErrFlag ;		//

} t_stcEpwm;


/* Global variables *********************************************************/

// Data fpr Epwm Modul
t_stcEpwm volatile tEpwmMode = {0,0};



//volatile unsigned long pwm_error_counter = 0;

/*!	\var pwm_error_flag
 * 	\brief Ist 0, wenn kein Fehler aufgetreten ist. 1 wenn ein Fehler (berlauf
 * 	der Zhlvariablen innerhalb einer gewissen Zeit -> Komplettabschaltung der
 * 	Ausgnge) aufgetreten ist.
 * 	Sie muss zunchst auf 1 gesetzt werden, bevor die PWM-Ausgnge wieder
 * 	herausgegeben werden.
 */
//volatile unsigned char pwm_error_flag = 0;




/* Functions ****************************************************************/



extern interrupt void FieldOrientedControl_ISR(void);

// Alle time critical functions must be copied at start time from Flash to Ram.
// Then they can run from Ram with lower runnung time.
// All this functions are linked to the "ramfuncs" section in Flash and load
// at the begin of main() to the internal Ram.
#pragma CODE_SECTION(HwiEpwm1TzInt, "ramfuncs");
//#pragma CODE_SECTION(cpu_timer1_isr, "ramfuncs");

//*****************************************************************************
/*! \brief Int. Handler, welcher bei einem zu hohem Strom aufgerufen wird
 *
 *	Der Strommesskomperator gibt ein Low-Signal aus, wenn ein berstrom
 *	aufgetreten ist.
 * 	Es wird in dieser Interruptroutine die Variable tEpwmMode.usErrorCnt
 * 	hochgezhlt, wenn diese innerhalb einer bestimmten Zeit einen bestimmten
 * 	Maximalen Wert (PWM_ERROR_MAX_COUNT) erreicht hat, wird die PWM
 * 	abgeschalten und das Flag der Variablen pwm_error_flag auf 1 gesetzt.
 */
//*****************************************************************************
interrupt void HwiEpwm1TzInt(void){
	
	// Clear the flags - we will continue to take
	// this interrupt until the TZ pin goes high
	//
	EALLOW;  
	EPwm1Regs.TZCLR.bit.INT = 1;
	EDIS;
	
	if(tEpwmMode.usErrorCnt == 0)
	{
		//Timer enable
		// Reload all counter register with period value:
		CpuTimer1Regs.TCR.bit.TRB = 1;
		CpuTimer1Regs.TCR.bit.TSS = 0;//Start Timer
	}
	// increment error counter, motor current more then 6,4A
	tEpwmMode.usErrorCnt++;
	if(tEpwmMode.usErrorCnt > PWM_ERROR_MAX_COUNT)
	{
		// motor current more then 6,4A for more then 50 periods, -> stop PWM
		PWMDisable();
		// Stop Timer
		CpuTimer1Regs.TCR.bit.TSS = 1;
		tEpwmMode.usErrFlag = 1;
		tEpwmMode.usErrorCnt = 0;
		// set diagnose state Pwm to Error OverCurrent
		*PtrDsMotorCon() = PWM_OVCUR;
		// signal for stopping the motor
		*PtrMeasState() = MM_STOP;
		asm(" nop");
	}
	else{
		EALLOW;
		EPwm1Regs.TZCLR.bit.CBC = 1;
		EDIS;
	}
	// Acknowledge this interrupt to receive more interrupts from group 2
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP2;
}

//*****************************************************************************
/*! \brief Oneshot Timerinterrupt (1ms). Dabei wird der Wert der
 * Variable pwm_error_counter zurcksetzt und der Timer gestoppt.
 *
 * Wenn nur kurz ein Fehler, z.B. durch Strungen aufgetreten ist, wird dieser
 * gelscht. [Siehe auch](@ref epwm_tzint_isr)
 */
//*****************************************************************************
/*
interrupt void cpu_timer1_isr(void)
{
	if(pwm_error_counter > 0)
	{
		pwm_error_counter = 0;
	CpuTimer1Regs.TCR.bit.TSS = 1;
	}
	CpuTimer1Regs.TCR.bit.TIF = 1;//Timerint Bit loeschen
	// The CPU acknowledges the interrupt.
	EDIS;
}
*/

//*****************************************************************************
/*! \brief Ausschalten der PWM-Impulse
 *
 *
 *	author david.berthold
 *	
 *	ber das Trip-Zone Modul wird ein One-Shot Trip event via Software
 * 	ausgelst, welcher die Ausgnge in die definierten Zustnde versetzt, in
 * 	diesem Fall alle auf L.
 *
 */
//*****************************************************************************
void PWMDisable(void){
	EALLOW;
	EPwm1Regs.TZFRC.bit.OST = 1;
	EPwm2Regs.TZFRC.bit.OST = 1;
	EPwm3Regs.TZFRC.bit.OST = 1;
	EPwm4Regs.TZFRC.bit.OST = 1;
	EPwm5Regs.TZFRC.bit.OST = 1;
	EPwm6Regs.TZFRC.bit.OST = 1;
	EDIS;
}

//*****************************************************************************
/*! \brief Einschalten der PWM-Impulse
 *
 *	author david.berthold
 *	
 *	ber das Trip-Zone Modul wird ein One-Shot Trip event via Software
 * 	ausgelst, welcher die Ausgnge in die definierten Zustnde versetzt, in
 * 	diesem Fall alle auf L.
 *
 */
//*****************************************************************************
void PWMEnable(void){
	//only enable, if no error was detected
	if(tEpwmMode.usErrFlag == 0){	
		EALLOW;
		EPwm2Regs.TZCLR.bit.OST = 1;
		EPwm1Regs.TZCLR.bit.OST = 1;
		EPwm4Regs.TZCLR.bit.OST = 1;
		EPwm3Regs.TZCLR.bit.OST = 1;
		EPwm6Regs.TZCLR.bit.OST = 1;
		EPwm5Regs.TZCLR.bit.OST = 1;
		EDIS;
	}
}

//*****************************************************************************
/*! \brief Start der Feldorientierten Regelung
 *
 *	author klaus.schneider
 *	
 *	Start der Interrupterzeugung des PWM-Moduls 1. In diesem Interrupt
 * 	wird die Berechnung der FOC durchgefhrt
 *
 */
//*****************************************************************************
void StartFOC(void){
	EPwm1Regs.ETCLR.all = 0xD;
	// Set Event-Trigger
	EPwm1Regs.ETSEL.bit.INTSEL = 0x1;	// Interrupt on counter equal to zero
	EPwm1Regs.ETSEL.bit.INTEN = 1;
	
	EPwm1Regs.ETPS.bit.INTPRD = FOC_BERECHNUNG_ALLE_X_MAL;	//Alle 3 Perioden Interrupt ausfhren
}

void StopFOC(void){
	EPwm1Regs.ETSEL.bit.INTEN = 0;
}



void TriggerGenEnable(void){
	   //ADCSOC wird als Triggerung fr ADS und IP genutzt
	EALLOW;
	//GPIO32 -> /ADCSOCA0 -> Convst ADS8329
	GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 3;
	
	//GPIO33 -> /ADCSOCB0 -> Trig IP2000
	GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 3;
	EDIS;
}

void TriggerGenDisable(void){
		   //ADCSOC wird als Triggerung fr ADS und IP genutzt
	EALLOW;
	//GPIO32 -> /ADCSOCA0 -> Convst ADS8329
	GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 0;
	
	//GPIO33 -> /ADCSOCB0 -> Trig IP2000
	GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 0;
	EDIS;
}

//*****************************************************************************
/*! \brief Initialisierung der PWM Module 1 bis 6
 *
 *	author david.berthold
 *	
 *	Die PWM-Module 1 bis 6 werden in separaten Funktionen initialisiert, dabei
 * 	wird jeweils nur der Ausgang A genutzt, weil die HRPWM nur auf Kanal A
 * 	verfgbar ist. Die Initialisierung der Interrupts fr die Fehlererkennung
 * 	erfolgt in dieser Funktion. Dabei wird der M_INT2 fr den Fault des
 * 	PWM-Moduls und der M_INT13 fr den Timer genutzt.
 */
//*****************************************************************************
void InitEPwmModul(void){
	EALLOW;
	//Fr die Syncro der Takte
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
	EDIS;
	tEpwmMode.usErrFlag = 0;
	tEpwmMode.usErrorCnt = 0;
	InitEPwm1();
	InitEPwm2();
	InitEPwm3();
	InitEPwm4();
	InitEPwm5();
	InitEPwm6();
	
	EALLOW;
	//Synchronisierung einschalten
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
	EDIS;
	
	EALLOW;
//#ifndef TODO
	PieVectTable.EPWM1_INT = &FieldOrientedControl_ISR;
//#endif
	EDIS;
	// Enable CPU INT3 which is connected to EPWM1-6 INT:
	IER |= M_INT3;
	//Enable Pie Interrupt
	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;
	
	//Tripzoneinterrupt Konfiguration
	 
	EALLOW;  // This is needed to write to EALLOW protected registers
	PieVectTable.EPWM1_TZINT = &HwiEpwm1TzInt;
	EDIS;    // This is needed to disable write to EALLOW protected registers
	
	// Enable CPU INT2 which is connected to EPWM1-6 TZINT:
	IER |= M_INT2;
	
	// Enable EPWM INTn in the PIE: Group 2 interrupt 1-3
	PieCtrlRegs.PIEIER2.bit.INTx1 = 1;
	

	// 26.11.2012, raus, in InitHwiInterrupts() implementiert
	//Timerinterrupt
	//EALLOW;  // This is needed to write to EALLOW protected registers
	//PieVectTable.XINT13 = &cpu_timer1_isr;
	//EDIS;
	//InitCpuTimers();//Init to a known state
	//ConfigCpuTimer(&CpuTimer1, CPU_FRQ/1000000, 1000);//2ms
	//CpuTimer1Regs.TCR.bit.TSS = 1;//Stop Timer
	//CpuTimer1Regs.TCR.bit.FREE = 0;//Stop Timer, wenn ein Breakpoint erreicht wird
	//CpuTimer1Regs.TCR.bit.SOFT = 0;
	
	// Enable CPU INT13 which is connected to Timer1
	//IER |= M_INT13;
}



/*!
\fn		float PtrEpwmErrCnt(void)
\brief  Read Epwm Error Counter
\author	Kster
\date	03.12.2012
\param
\return	void
\details Changes:
*/
volatile Uint16* PtrEpwmErrCnt(void)
{
	return &tEpwmMode.usErrorCnt;
}








//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 1
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Highsidemosfet (MOT-HB1-H) verbunden.
 * 	Es wird nicht die hardwaremssige Todzeiteinfgung genutzt, weil dadurch 
 * 	eine Unsymmetrie entsteht.
 *
 */
//*****************************************************************************
void InitEPwm1(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm1Regs.TZCTL.bit.TZA = TZ_FORCE_LO;		//Im Fehlerfall soll Ausgang auf 0 gesetzt werden
	EPwm1Regs.TZSEL.bit.CBC1 = TZ_ENABLE;		//Continiusmode: Wenn Timer den Wert 0 erreicht werden PWM-Impulse wieder herausgegeben
	
	// Enable TZ interrupt
	EPwm1Regs.TZEINT.bit.CBC = 1;//todo war 1
	
	//PWM ausschalten mittel Tripzone
	EPwm1Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm1Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm1Regs.TBPHS.half.TBPHS = 0x0000;            // Phase is 0
	EPwm1Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm1Regs.TBSTS.bit.CTRDIR = 1;					//Counting up -> kann entfallen weil updown-Mode

	EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;			// Enable phase loading
	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope

	EPwm1Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;		// Sync down-stream module
	EPwm1Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 0 (H-Side)

	// Set Counter-Compare
	EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;	// Load registers every ZERO
	//EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm1Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm1Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm1Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm1Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm1Regs.AQCTLA.bit.CAU = AQ_SET;				// Set PWM4A on Zero
	EPwm1Regs.AQCTLA.bit.CAD = AQ_CLEAR;
	EPwm1Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm1Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm1Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm1Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

	EPwm1Regs.AQCTLB.bit.CBU = AQ_NO_ACTION;
	EPwm1Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm1Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm1Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm1Regs.AQCTLB.bit.ZRO = AQ_NO_ACTION;
	EPwm1Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;

	// Set Dead-Band Generator
	EPwm1Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm1Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm1Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm1Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm1Regs.HRCNFG.bit.EDGMODE = HR_REP;			//Beide Flanken werden beeinflusst
	EDIS;

	/*
	// Set Event-Trigger
	EPwm1Regs.ETSEL.all=0x0A;      // Interrupt on PRD
	// EPwm1Regs.ETPS.all=1;
	EPwm1Regs.ETPS.all=0x0F;      // Interrupt on PRD
	
	EPwm1Regs.ETFLG.all=0;
	EPwm1Regs.ETCLR.all=0;
	EPwm1Regs.ETFRC.all=0;
	
	// EPwm1Regs.PCCTL.all=0;
	
	// EPwm1Regs.TBCTL.all=0x0010+TBCTLVAL; // Enable Timer
	// EPwm1Regs.TBPRD=SP;
	*/
	
	
	//Enable Counter
	EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down

}

//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 2
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Lowsidemosfet (MOT-HB1-L) verbunden.
 * 	Siehe auch InitEPwm1
 *
 */
//*****************************************************************************
void InitEPwm2(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm2Regs.TZCTL.bit.TZA = TZ_FORCE_LO;
	EPwm2Regs.TZCTL.bit.TZB = TZ_FORCE_HI;
	EPwm2Regs.TZSEL.bit.CBC1 = TZ_ENABLE;
	
	//PWM ausschalten mittel TZ
	EPwm2Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm2Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm2Regs.TBPHS.half.TBPHS = 0x0002;            // Phase is 0
	EPwm2Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm2Regs.TBSTS.bit.CTRDIR = 1;

	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;			// Enable phase loading
	EPwm2Regs.TBCTL.bit.PHSDIR = 1;					//Count Up on Sync Event
	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope

	EPwm2Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		// 0 -> sync flow-through
	EPwm2Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 1 (L-Side)

	// Set Counter-Compare
	EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	//EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm2Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm2Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm2Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm2Regs.AQCTLA.bit.CAU = AQ_CLEAR;              // Set PWM2A
	EPwm2Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm2Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm2Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm2Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm2Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH
	EPwm2Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm2Regs.AQCTLB.bit.CBD = AQ_CLEAR;
	EPwm2Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.ZRO = AQ_SET;
	EPwm2Regs.AQCTLB.bit.PRD = AQ_SET;
#else
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_DOWN
	EPwm2Regs.AQCTLB.bit.CBU = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.CBD = AQ_CLEAR;
	EPwm2Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.ZRO = AQ_SET;
	EPwm2Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;
#endif
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_UP
	EPwm2Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm2Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.ZRO = AQ_NO_ACTION;
	EPwm2Regs.AQCTLB.bit.PRD = AQ_SET;
#endif
#endif

	// Set Dead-Band Generator
	EPwm2Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm2Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm2Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm2Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm2Regs.HRCNFG.bit.EDGMODE = HR_FEP;			//Beide Flanken werden beeinflusst
	EDIS;
	
	//Generate the ADS-Conv ber ADCSOC (SPRUG04A P.108) at GPIO32
	EPwm2Regs.ETSEL.bit.SOCAEN = 1;					//Enable SOCA fr EPWM2
	EPwm2Regs.ETSEL.bit.SOCASEL = 0x2;				//Generate ADC Trigger at TBCTR = TBPRD
	EPwm2Regs.ETPS.bit.SOCAPRD = 1;					//Jedes mal Triggern
	EPwm2Regs.ETCLR.bit.SOCA = 1;					//Event lschen
		
	//Enable Counter
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down
	
}

//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 3
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Highsidemosfet (MOT-HB2-H) verbunden.
 * 	Siehe auch InitEPwm1
 *
 */
//*****************************************************************************
void InitEPwm3(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm3Regs.TZCTL.bit.TZA = TZ_FORCE_LO;
	EPwm3Regs.TZCTL.bit.TZB = TZ_FORCE_LO;
	EPwm3Regs.TZSEL.bit.CBC1 = TZ_ENABLE;
	
	//PWM ausschalten mittel TZ
	EPwm3Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm3Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm3Regs.TBPHS.half.TBPHS = 0x0002;            // Phase is 0
	EPwm3Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm3Regs.TBSTS.bit.CTRDIR = 1;

	EPwm3Regs.TBCTL.bit.PHSEN = TB_ENABLE;			// Enable phase loading
	EPwm3Regs.TBCTL.bit.PHSDIR = 1;					//Count Up on Sync Event
	EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope


	EPwm3Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		// 0 -> sync flow-through
	EPwm3Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 0 (H-Side) 

	// Set Counter-Compare
	EPwm3Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	//EPwm3Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm3Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm3Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm3Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm3Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm3Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm3Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm3Regs.AQCTLA.bit.CAU = AQ_SET;              // Set PWM3A
	EPwm3Regs.AQCTLA.bit.CAD = AQ_CLEAR;
	EPwm3Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm3Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm3Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm3Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

	EPwm3Regs.AQCTLB.bit.CBU = AQ_SET;            // Set PWM3B
	EPwm3Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm3Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm3Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm3Regs.AQCTLB.bit.ZRO = AQ_CLEAR;
	EPwm3Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;

	// Set Dead-Band Generator
	EPwm3Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm3Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm3Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm3Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm3Regs.HRCNFG.bit.EDGMODE = HR_REP;			//Beide Flanken werden beeinflusst
	EDIS;
	
	//Generate the IP2000-Trig-Signal with ADCSOC (SPRUG04A P.108) at GPIO33
	EPwm3Regs.ETSEL.bit.SOCBEN = 1;					//Enable SOCB for EPWM3
	EPwm3Regs.ETSEL.bit.SOCBSEL = 0x2;				//Generate ADC Trigger at TBCTR = TBPRD
	EPwm3Regs.ETPS.bit.SOCBPRD = 1;					//Jedes mal Triggern
	EPwm3Regs.ETCLR.bit.SOCB = 1;					//Event lschen
	
	//Enable Counter
	EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down
}

//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 4
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Highsidemosfet (MOT-HB2-L) verbunden.
 * 	Siehe auch InitEPwm1
 *
 */
//*****************************************************************************
void InitEPwm4(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm4Regs.TZCTL.bit.TZA = TZ_FORCE_LO;
	EPwm4Regs.TZSEL.bit.CBC1 = TZ_ENABLE;
	
	//PWM ausschalten mittel TZ
	EPwm4Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm4Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm4Regs.TBPHS.half.TBPHS = 0x0002;            // Phase is 0
	EPwm4Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm4Regs.TBSTS.bit.CTRDIR = 1;

	EPwm4Regs.TBCTL.bit.PHSEN = TB_ENABLE;			// Enable phase loading
	EPwm4Regs.TBCTL.bit.PHSDIR = 1;					//Count Up on Sync Event
	EPwm4Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm4Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope

	EPwm4Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm4Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		// 0 -> sync flow-through
	EPwm4Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 1 (L-Side) 

	// Set Counter-Compare
	EPwm4Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	//EPwm4Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm4Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm4Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm4Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm4Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm4Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm4Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm4Regs.AQCTLA.bit.CAU = AQ_CLEAR;              // Set PWM4A on Zero
	EPwm4Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm4Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm4Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm4Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm4Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

	EPwm4Regs.AQCTLB.bit.CBU = AQ_NO_ACTION;            // Set PWM4B
	EPwm4Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm4Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm4Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm4Regs.AQCTLB.bit.ZRO = AQ_NO_ACTION;
	EPwm4Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;

	// Set Dead-Band Generator
	EPwm4Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm4Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm4Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm4Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm4Regs.HRCNFG.bit.EDGMODE = HR_FEP;			//Beide Flanken werden beeinflusst
	EDIS;
	
	//Enable Counter
	EPwm4Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down
}

//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 5
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Highsidemosfet (MOT-HB3-H) verbunden.
 * 	Siehe auch InitEPwm1
 *
 */
//*****************************************************************************
void InitEPwm5(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm5Regs.TZCTL.bit.TZA = TZ_FORCE_LO;
	EPwm5Regs.TZSEL.bit.CBC1 = TZ_ENABLE;
	
	//PWM ausschalten mittel TZ
	EPwm5Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm5Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm5Regs.TBPHS.half.TBPHS = 0x0002;            // Phase is 2
	EPwm5Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm5Regs.TBSTS.bit.CTRDIR = 1;

	EPwm5Regs.TBCTL.bit.PHSEN = TB_ENABLE;			// Enable phase loading
	EPwm5Regs.TBCTL.bit.PHSDIR = 1;					//Count Up on Sync Event
	EPwm5Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm5Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope

	EPwm5Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm5Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		// 0 -> sync flow-through
	EPwm5Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 0 (H-Side)

	// Set Counter-Compare
	EPwm5Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	//EPwm5Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm5Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm5Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm5Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm5Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm5Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm5Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm5Regs.AQCTLA.bit.CAU = AQ_SET;              // Set PWM4A on Zero
	EPwm5Regs.AQCTLA.bit.CAD = AQ_CLEAR;
	EPwm5Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm5Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm5Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm5Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

	EPwm5Regs.AQCTLB.bit.CBU = AQ_NO_ACTION;            // Set PWM4B on Zero
	EPwm5Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm5Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm5Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm5Regs.AQCTLB.bit.ZRO = AQ_NO_ACTION;
	EPwm5Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;

	// Set Dead-Band Generator
	EPwm5Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm5Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm5Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm5Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm5Regs.HRCNFG.bit.EDGMODE = HR_REP;			//Beide Flanken werden beeinflusst
	EDIS;
	
	//Enable Counter
	EPwm5Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down
}

//*****************************************************************************
/*! \brief Initialisierung des PWM-Moduls 6
 *
 *	author david.berthold
 *	
 *	Dieses PWM-Modul ist mit dem Highsidemosfet (MOT-HB3-L) verbunden.
 * 	Siehe auch InitEPwm1
 *
 */
//*****************************************************************************
void InitEPwm6(void)
{
	//Trip-Zone Submodul (Fehlererkennung)
	EALLOW;
	EPwm6Regs.TZCTL.bit.TZA = TZ_FORCE_LO;
	EPwm6Regs.TZSEL.bit.CBC1 = TZ_ENABLE;
	
	//PWM ausschalten mittel TZ
	EPwm6Regs.TZFRC.bit.OST = 1;
	EDIS;
	
	// Set Time-Base
	EPwm6Regs.TBPRD = EPWM_COUNTER_PERIOD - 1;		// Set timer period
	EPwm6Regs.TBPHS.half.TBPHS = 0x0002;            // Phase is 0
	EPwm6Regs.TBCTR = 0x0000;                       // Clear counter
	EPwm6Regs.TBSTS.bit.CTRDIR = 1;

	EPwm6Regs.TBCTL.bit.PHSEN = TB_ENABLE;			// Enable phase loading
	EPwm6Regs.TBCTL.bit.PHSDIR = 1;					//Count Up on Sync Event
	EPwm6Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;		// Clock ratio to SYSCLKOUT -> HSPCLKDIV = 1
	EPwm6Regs.TBCTL.bit.CLKDIV = TB_DIV1;			// Slow so we can observe on the scope


	EPwm6Regs.TBCTL.bit.PRDLD = TB_SHADOW;			// 0
	EPwm6Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;		// 0 -> sync flow-through
	EPwm6Regs.TBCTL.bit.FREE_SOFT =  0x01;			//On Breakpoint, stop when TBCTR = 0x0000 -> OUTPUT = 0 (L-Side) 

	// Set Counter-Compare
	EPwm6Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	//EPwm6Regs.CMPCTL.bit.SHDWAMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm6Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	//EPwm6Regs.CMPCTL.bit.SHDWBMODE = CC_IMMEDIATE;	// 1 -> ?
	EPwm6Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;	// load on CTR=Zero
	EPwm6Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;	// load on CTR=Zero

	EPwm6Regs.CMPA.half.CMPA = 0;					// maybe comment out
	//EPwm6Regs.CMPA.half.CMPAHR = 0;				// maybe comment out

	// Set Action-Qualifier
	EPwm6Regs.AQCTLA.bit.CAU = AQ_CLEAR;			// Set PWM4A on Zero
	EPwm6Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm6Regs.AQCTLA.bit.CBU = AQ_NO_ACTION;
	EPwm6Regs.AQCTLA.bit.CBD = AQ_NO_ACTION;
	EPwm6Regs.AQCTLA.bit.ZRO = AQ_NO_ACTION;
	EPwm6Regs.AQCTLA.bit.PRD = AQ_NO_ACTION;

	EPwm6Regs.AQCTLB.bit.CBU = AQ_NO_ACTION;            // Set PWM4B on Zero
	EPwm6Regs.AQCTLB.bit.CBD = AQ_NO_ACTION;
	EPwm6Regs.AQCTLB.bit.CAU = AQ_NO_ACTION;
	EPwm6Regs.AQCTLB.bit.CAD = AQ_NO_ACTION;
	EPwm6Regs.AQCTLB.bit.ZRO = AQ_NO_ACTION;
	EPwm6Regs.AQCTLB.bit.PRD = AQ_NO_ACTION;

	// Set Dead-Band Generator
	EPwm6Regs.DBCTL.bit.OUT_MODE = DB_DISABLE;			// Deadband disabled
	EPwm6Regs.DBCTL.bit.POLSEL = DB_ACTV_HI;
	
	//Einstellen des HRPWM-Moduls
	EALLOW;
	EPwm6Regs.HRCNFG.bit.HRLOAD = HR_CTR_ZERO;		//Shadow Load at zero
	EPwm6Regs.HRCNFG.bit.CTLMODE = HR_CMP;			//HRWPM works at duty control mode
	EPwm6Regs.HRCNFG.bit.EDGMODE = HR_FEP;			//Beide Flanken werden beeinflusst
	EDIS;
	
	//Enable Counter
	EPwm6Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;	// Count up and down
}

void InitEPwm1Gpio(void)
{
   EALLOW;
   
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

	GpioCtrlRegs.GPAPUD.bit.GPIO0 = 1;    // Disable pull-up on GPIO0 (EPWM1A)   
   
/* Configure ePWM-1 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM1 functional pins.
// Comment out other unwanted lines.

//    GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 0;   // Configure GPIO0 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO0 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1;   // Configure GPIO0 as EPWM1A
   
    EDIS;
}

void InitEPwm2Gpio(void)
{
   EALLOW;
	
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

	GpioCtrlRegs.GPAPUD.bit.GPIO2 = 1;    // Disable pull-up on GPIO2 (EPWM2A)

/* Configure ePWM-2 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM2 functional pins.
// Comment out other unwanted lines.

//    GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 0;   // Configure GPIO2 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO2 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 1;   // Configure GPIO2 as EPWM2A
//#if(PCB == TFS_PCB)
//	GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 1;   // Configure GPIO3 as EPWM2B
//	GpioCtrlRegs.GPAQSEL1.bit.GPIO3 = 0x3;
//#endif
   
    EDIS;
}

void InitEPwm3Gpio(void)
{
   EALLOW;
   
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAPUD.bit.GPIO4 = 1;    // Disable pull-up on GPIO4 (EPWM3A)
	GpioCtrlRegs.GPAPUD.bit.GPIO5 = 1;	// Disable pull-up on GPIO5 (EPWM3B)
       
/* Configure ePWM-3 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM3 functional pins.
// Comment out other unwanted lines.

//    GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 0;   // Configure GPIO4 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO4 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1;   // Configure GPIO4 as EPWM3A

    EDIS;
}

void InitEPwm4Gpio(void)
{
   EALLOW;
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAPUD.bit.GPIO6 = 1;    // Disable pull-up on GPIO6 (EPWM4A)
    //GpioCtrlRegs.GPAPUD.bit.GPIO7 = 1;    // Disable pull-up on GPIO7 (EPWM4B)
    
/* Configure ePWM-4 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM4 functional pins.
// Comment out other unwanted lines.

//	GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 0;   // Configure GPIO6 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 1;   // Configure GPIO6 as EPWM4A
	//GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 1;   // Configure GPIO7 as EPWM4B
	
    EDIS;
}  

void InitEPwm5Gpio(void)
{
   EALLOW;
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

	GpioCtrlRegs.GPAPUD.bit.GPIO8 = 1;    // Disable pull-up on GPIO8 (EPWM5A)

/* Configure ePWM-5 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM5 functional pins.
// Comment out other unwanted lines.

//	GpioCtrlRegs.GPAMUX1.bit.GPIO8 = 0;   // Configure GPIO8 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO8 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO8 = 1;   // Configure GPIO8 as EPWM5A
	
    EDIS;
}

void InitEPwm6Gpio(void)
{
   EALLOW;

/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAPUD.bit.GPIO10 = 1;    // Disable pull-up on GPIO10 (EPWM6A)
    
/* Configure ePWM-6 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be ePWM6 functional pins.
// Comment out other unwanted lines.

//	GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 0;   // Configure GPIO10 as GPIO
//	GpioDataRegs.GPACLEAR.bit.GPIO10 = 1;	//LOW
	GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;   // Configure GPIO10 as EPWM6A
	
    EDIS;
}  

//---------------------------------------------------------------------------
// Example: InitEPwmSyncGpio: 
//---------------------------------------------------------------------------
// This function initializes GPIO pins to function as ePWM Synch pins
//

void InitEPwmSyncGpio(void)
{

   EALLOW;

/* Configure EPWMSYNCI  */
   
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

   GpioCtrlRegs.GPAPUD.bit.GPIO6 = 0;    // Enable pull-up on GPIO6 (EPWMSYNCI)
// GpioCtrlRegs.GPBPUD.bit.GPIO32 = 0;   // Enable pull-up on GPIO32 (EPWMSYNCI)    

/* Set qualification for selected pins to asynch only */
// This will select synch to SYSCLKOUT for the selected pins.
// Comment out other unwanted lines.

   GpioCtrlRegs.GPAQSEL1.bit.GPIO6 = 0;   // Synch to SYSCLKOUT GPIO6 (EPWMSYNCI)
// GpioCtrlRegs.GPBQSEL1.bit.GPIO32 = 0;  // Synch to SYSCLKOUT GPIO32 (EPWMSYNCI)    

/* Configure EPwmSync pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be EPwmSync functional pins.
// Comment out other unwanted lines.   

   GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 2;    // Enable pull-up on GPIO6 (EPWMSYNCI)
// GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 2;   // Enable pull-up on GPIO32 (EPWMSYNCI)    



/* Configure EPWMSYNC0  */

/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

// GpioCtrlRegs.GPAPUD.bit.GPIO6 = 0;    // Enable pull-up on GPIO6 (EPWMSYNC0)
   GpioCtrlRegs.GPBPUD.bit.GPIO33 = 0;   // Enable pull-up on GPIO33 (EPWMSYNC0)    

// GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 3;    // Enable pull-up on GPIO6 (EPWMSYNC0)
   GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 2;   // Enable pull-up on GPIO33 (EPWMSYNC0)    

}



//---------------------------------------------------------------------------
// Example: InitTzGpio: 
//---------------------------------------------------------------------------
// This function initializes GPIO pins to function as Trip Zone (TZ) pins
//
// Each GPIO pin can be configured as a GPIO pin or up to 3 different
// peripheral functional pins. By default all pins come up as GPIO
// inputs after reset.  
// 

void InitTzGpio(void)
{
   EALLOW;
   
/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user. 
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.
   GpioCtrlRegs.GPAPUD.bit.GPIO12 = 0;    // Enable pull-up on GPIO12 (TZ1)
//   GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;    // Enable pull-up on GPIO13 (TZ2)
//   GpioCtrlRegs.GPAPUD.bit.GPIO14 = 0;    // Enable pull-up on GPIO14 (TZ3)
//   GpioCtrlRegs.GPAPUD.bit.GPIO15 = 0;    // Enable pull-up on GPIO15 (TZ4)

//   GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;    // Enable pull-up on GPIO16 (TZ5)
// GpioCtrlRegs.GPAPUD.bit.GPIO28 = 0;    // Enable pull-up on GPIO28 (TZ5)

//   GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;    // Enable pull-up on GPIO17 (TZ6) 
// GpioCtrlRegs.GPAPUD.bit.GPIO29 = 0;    // Enable pull-up on GPIO29 (TZ6)  
   
/* Set qualification for selected pins to asynch only */
// Inputs are synchronized to SYSCLKOUT by default.  
// This will select asynch (no qualification) for the selected pins.
// Comment out other unwanted lines.

   GpioCtrlRegs.GPAQSEL1.bit.GPIO12 = 3;  // Asynch input GPIO12 (TZ1)
   //GpioCtrlRegs.GPAQSEL1.bit.GPIO12 = 0x01;  // Synch 3 samples input to GPIO12 (TZ1)
   //GpioCtrlRegs.GPAQSEL1.bit.GPIO12 = 0x10;  // Synch 6 samples input to GPIO12 (TZ1)
//   GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 3;  // Asynch input GPIO13 (TZ2)
//   GpioCtrlRegs.GPAQSEL1.bit.GPIO14 = 3;  // Asynch input GPIO14 (TZ3)
//   GpioCtrlRegs.GPAQSEL1.bit.GPIO15 = 3;  // Asynch input GPIO15 (TZ4)
//
//   GpioCtrlRegs.GPAQSEL2.bit.GPIO16 = 3;  // Asynch input GPIO16 (TZ5)
// GpioCtrlRegs.GPAQSEL2.bit.GPIO28 = 3;  // Asynch input GPIO28 (TZ5)

//   GpioCtrlRegs.GPAQSEL2.bit.GPIO17 = 3;  // Asynch input GPIO17 (TZ6) 
// GpioCtrlRegs.GPAQSEL2.bit.GPIO29 = 3;  // Asynch input GPIO29 (TZ6)  

   
/* Configure TZ pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be TZ functional pins.
// Comment out other unwanted lines.   
   GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 1;  // Configure GPIO12 as TZ1
//   GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;  // Configure GPIO13 as TZ2
//   GpioCtrlRegs.GPAMUX1.bit.GPIO14 = 1;  // Configure GPIO14 as TZ3
//   GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 1;  // Configure GPIO15 as TZ4
//
//   GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 3;  // Configure GPIO16 as TZ5
// GpioCtrlRegs.GPAMUX2.bit.GPIO28 = 3;  // Configure GPIO28 as TZ5

//   GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 3;  // Configure GPIO17 as TZ6               
// GpioCtrlRegs.GPAMUX2.bit.GPIO29 = 3;  // Configure GPIO29 as TZ6  

   EDIS;
}

// This function initializes GPIO pins to function as ePWM pins
//
// Each GPIO pin can be configured as a GPIO pin or up to 3 different
// peripheral functional pins. By default all pins come up as GPIO
// inputs after reset.  
// 

void InitEPwmGpio(void)
{
   InitEPwm1Gpio();
   InitEPwm2Gpio();
   InitEPwm3Gpio();
   InitEPwm4Gpio();
   InitEPwm5Gpio();
   InitEPwm6Gpio();
   
	EALLOW;
	//GPIO32 -> /ADCSOCA0 -> Convst ADS8329
	GpioCtrlRegs.GPBPUD.bit.GPIO32 = 0;
	
	//GPIO33 -> /ADCSOCB0 -> Trig IP2000
	GpioCtrlRegs.GPBPUD.bit.GPIO33 = 0;
	EDIS;

   //Fault Eingangspininit
   InitTzGpio();
}
//*****************************************************************************
/*! @}
 *	Close the Doxygen group.
 */
//*****************************************************************************
