/*!
\addtogroup Main
\file	main.c
\brief	Visco Vision K1 Main-Program. Initialization of Hardware and
		Firmware modules. Copy of Ram-Functions from Flash to Ram.
		Haake BIOS for tasks with lower priorities.


\author	Koester
\date	01.10.2011

\details
	changes:

*/

/* Include files ************************************************************/
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
//#include "C28x_FPU_FastRTS.h"
#include "Device.h"
//#define GLOBAL_DECL
//#include "global.h"
#include "timer.h"
#include "McbspDma.h"
#include "ADS8329.h"
#include "Epwm.h"
#include "SpiA.h"
#include "Uc1dsp.h"
#include "version.h"	
#include "IntSvc.h"
#include "Ip2000.h"
#include "angle.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "Measmode.h"
#include "control.h"
#include "Crmode.h"
#include "CsFastOsc.h"
#include "Csmode.h"
#include "fosc.h"
#include "correction.h"
#include "control.h"												 
#include "SsiComm.h"
#include "Diagnos.h"
#include "Test.h"
#include "msc.h"
#include "filt.h"

/* Macro definitions ********************************************************/
#define TSK_50uS	5
#define TSK_1MS	10
#define TSK_5MS	50
#define TSK_100MS	1000
#define TSK_250MS	2500
#define TSK_1000MS	10000
#define TSK_5000MS	50000


/* Types ********************************************************************/

// Datenstrukturen f�r Timer-Interupte und Tasks, vornehmlich f�r Diagnose
t_stcHwiData tRootTask;

/* Global variables *********************************************************/

// Function pointer RootTask HAAKE-BIOS
void (*PtrRootTask)(void);

/* external Function prototypes**********************************************/
extern void FocInit(void);

extern void Swi05_FastOscInit(void);

/* internal Function prototypes**********************************************/

// HAAKE-BIOS Root Task State Machine function prototypes
void RootTaskA0(void);	//state RootTaskA0
void RootTaskA1(void);	//state A1
//void RootTaskA2(void);	//state A2

// Used for running BackGround in flash, and ISR in RAM
//extern Uint16 *RamfuncsLoadStart, *RamfuncsLoadEnd, *RamfuncsRunStart;


/* Functions ****************************************************************/

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - starts here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void main(void)
{
	//ErrorCodes returned_error_code = NoError;
	
	//=================================================================================
	//	INITIALISATION - General - System
	//=================================================================================
		
	// Device Live-Support PLL, PieVectTable, Sysclks, Gpio's
	DeviceInit();

#ifdef FLASH
	// Only used if running from FLASH
	// the variable FLASH is defined by the compiler with -d FLASH		
	// Copy time critical code and Flash setup code to RAM
	// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
	// symbols are created by the linker. Refer to the linker files.
	// F28335_FLASH_VV.CMD -> Sections -> ramfuncs 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);
	// Call Flash Initialization to setup flash waitstates
	// This function must reside in RAM
	InitFlash();
#endif //(FLASH)
                                             
	// EDC define Gpios for EPWM Modul
	InitEPwmGpio();
	// EDC define Gpios for ADS8329
	InitMcbspbGpio();
	// EDC define Gpios for Spi-A
	InitSpiAGpio();
	//
	InitEPwmModul();
	//Einstellung des Komperatorwertes mit Todzeit
	EPwm1Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 + PWM_TOTZEIT_2;//H
	EPwm2Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 - PWM_TOTZEIT_2;//L
	EPwm3Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 + PWM_TOTZEIT_2;
	EPwm4Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 - PWM_TOTZEIT_2;
	EPwm5Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 + PWM_TOTZEIT_2;
	EPwm6Regs.CMPA.half.CMPA = EPWM_COUNTER_PERIOD/2 - PWM_TOTZEIT_2;
	
	//Generierung des Conv-Start f�r ADS erfolgt in PWM2CMPB
	ADS8329_SET_TRIGGER(EPWM_COUNTER_PERIOD - 10);
	
	//Start der �bertragung f�r den IP2000 erfolgt �ber die PWM3 CompB
	IP2000_SET_TRIGGER(EPWM_COUNTER_PERIOD - 10);

	// Diagnosis delete states
	InitDiagnosStates();
	// set CPU diagnosis to init
	*PtrDsCpu() = CPU_INIT;
	// CPU, Version, other data
	InitSystemInfo();
	
	// Timer are running after Reset, therefor STOP
	StopCpuTimer0();
	StopCpuTimer1();
	StopCpuTimer2();
	// Initialize the Cpu Timers 0-2
	InitCpuTimers();

	// Initialize Interrupt-Functions, vector-Table and others
	//InitHwiInterrupts();

	// Device and Sensor init.
	DeviceSensorInit();
	// Winkelmessung initialisieren
	AngleSetCondCalc(DEEP_25);

	// initialize Omega calculation, Filter 250Hz, Sampling-Time 2ms, MAV-Deep 50
	OmegaSetCondCalc(F250HZ, ST_200US, TIME_500MS);
//	OmegaSetCondCalc(F50HZ, ST_200US, TIME_100MS);
	// and reset Filter with 0.0 values
	OmegaInitFilter(0.0);
	OmegaResetAccelaration();
	// Filter Torque initialisieren 
	TorqueSetCondCalc(F250HZ, ST_200US, TIME_500MS);
	//TorqueSetCondCalc(F50HZ, ST_200US, TIME_100MS);
	// und Filter zur�cksetzen auf Null
	// todo 22.01.16 TorqueInitFilter(0.0);
	// Korrekturen vorbelegen
	SetCondCorrections();	

	// Init. SSI Interface als Slave
	SsiInit();

	// before touching DMA-Modul, do Hardware-Reset	
	//DmaHardReset();
	// MCBSP init. as SPI-Slave with DMA-Transfer (data from/to �C1)

	InitMcbspADmaTransfer();

	// Messmodi Rotation und im allgemeinen initialisieren, Vars und Filter etc.
	MeasmodeInit();
	
	// Interface function to configure the fast oscillation device object
	InitFastOscillationData();

	// take care, includes a DMA Hardware-Reset
	FocInit();

	// Init AD-Convertrs of motor-control, and store message in diagnose modul
	*PtrDsMotorCon() = ADCInit();

	// Init GC-IP2000, and store message in diagnose modul
	*PtrDsIP() = Ip2000Init();

	#ifdef RAUS
	if( (*PtrDsMotorCon() != 0) || (*PtrDsIP() != 0) )
	{
		// Next two lines for debug only to halt the processor here
		// Remove after inserting ISR Code
		asm("          ESTOP0");
		//GpioDataRegs.GPBTOGGLE.bit.GPIO49 = 1;
		//NOP for Breakpoint
		asm(" nop");
		for(;;);
	}
	#endif
		
	
	// Initialize Interrupt-Functions, vector-Table and others
	InitHwiInterrupts();

	//Min. 2 PWM-Taktperioden warten bis die einzelnen Z�hler sich aufeinander
	//synchronisert haben -> Sie bekommen nur bei PWM1 CTR=0x0000 einen Impuls
	DELAY_US(1000000*1/EPWM_FRQ*10);
	TriggerGenEnable();
	
	// Start FOC later, after having waited 1 sec., Koester
	PWMEnable();
	DELAY_US(60);
	// finally start the FOC
	StartFOC();

	// Start Idle Timer-2 for Root-Task with 0.1ms sampling time!
	Timer2InitStart(100);
	// Timer period definitions found in PeripheralHeaderIncludes.h

	// ********* RAKOS RTOS State-machine init ***************************
	// call several Init functions only after Start, use semaphore

	tRootTask.usSemaphor = 0;	// 14.04.15 war 1, start aber jetzt durch �C-1 Kommando

	// start with RootTaskA0
	PtrRootTask = &RootTaskA0;
	// reset the most important Counter for the Task State machine
	tRootTask.ulCnt = 0;
	// ********* Tasks State-machine init ********************************

	// set CPU diagnosis to init
	*PtrDsCpu() = CPU_RUN;
	// Test Cases

	// ********* RAKOS RTOS Background Loop ******************************
	//infinite loop
	for(;;)
	{
		// State machine entry & exit point
		asm(" nop");
		(*PtrRootTask)();	// jump to the Root-Task
	}
	// ********* RAKOS RTOS Background Loop ******************************
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - ends here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



//=================================================================================
//	STATE-MACHINE SEQUENCING AND SYNCRONIZATION with Root-Task
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------

/*!
\brief		Root Task A0 Real-Time background functions.
			RootTaskA01 is the Main Function of the infinit loop at end of main().
			This functions is polling the always running Timer2 which runs with 100�s.
			When Timer-2 overflow is detected, the RootTask-0 is doing the written functions
			as e.G. the MSC Detection Control if switched ON!
			This means all functions within this if-case are done all 100�s.
			At the end the RootTask-A1 is called, to get more different sampling periods.

\fn			void RootTaskA0(void)
\author		K�ster, Ralf
\date		13.04.2012
\param[IN]	
\return		
\details	Changes:	
*/

void RootTaskA0(void)		// via Timer-2, all 0.1ms
{
	// check Timer Overflow if yes, then 0.1ms are over
	if(CpuTimer2Regs.TCR.bit.TIF == 1)
	{
		// ****** here all functions are done all 100�s *****
		// clear flag Timer-2 overflow
		CpuTimer2Regs.TCR.bit.TIF = 1;
		
		// 01.10.13 now in HwiTimer2(), check for new command from master and do....
		//SwiMasterCommand();
		
		// ********** MSC functions **********
		// in case of state start MSC table calculation
		switch(*PtrMscState())
		{
			case MSC_START_AQU:
				//*PtrMscState() = FALSE;
				*PtrMscState() = MSC_DATA_AQU;
				MscDatAqInit(&tMscData);
				// update State in Meas-Data, data aqu. is running
				*PtrMeasActDataMode() = MSC_DATA_AQU;
			break;
			case MSC_START_ANALYSE:
				//*PtrMscState() = FALSE;
				// update State in Meas-Data for Master
				//*PtrMeasActDataMode() = MSC_DATA_CAL;
				MscTblCalc(&tMscData);
				// Add result to Msc-Avg-Buffer
				MscTblAddAvgBuffer();
				// increment AVG-Counter for averaging the MSC detection
				*PtrMscAvgCnt() = *PtrMscAvgCnt() +1;
				// Analyse MSC-Detection result, search Amplitude
				//*PtrSysMscAmplitude() = TblSearchMinMax(&tMscData);
				// after 5 times MSC detection, calculate AVG value
				if( *PtrMscAvgCnt() == MSC_AVG)
				{
					// calculate the average values
					CorrMscTblAvgCalc();
					// update State in Meas-Data for Master
					// Analyse MSC-Detection result, search Amplitude
					*PtrSysMscAmplitude() = TblSearchMinMax(&tMscData);
					*PtrMeasActDataMode() = MSC_DATA_AQU_RDY;
					*PtrMscState() = FALSE;
					//*PtrMeasActDataMode() = MSC_DATA_CAL_RDY;
				}
				else
				{
					// init MSC data aquisition once more
					MscDatAqInit(&tMscData);
					// semaphore for MscDatAqInit() in main.c
					*PtrMscState() = MSC_DATA_AQU;
					// update State in Meas-Data, data aqu. is running
					//*PtrMeasActDataMode() = MSC_DATA_AQU;
				}
			break;
			case MSC_END:
				// Info to Meas. Control
				*PtrMeasActDataMode() = MSC_DATA_CAL_RDY;
				// Info to master data structur
				*PtrMeasDataState() = MSC_DATA_CAL_RDY;
			break;
			default:
			break;
		}
		// ********* End MSC functions **********

		// go next to A1 
		PtrRootTask = &RootTaskA1;
	}
}


/*!
\brief		Root Task A1 Real-Time background functions
			RootTaskA1 is a subroutine from RootTaskA0 which is called at every end
			of the RootTaskA0 which is done all 100�s.
			The RootTaskA1 counts up the tRootTask.ulCnt from zero to 50000 which means, that
			this counter makes a time-schedule for functions with low priority.
			With the modulo function the tRootTask.ulCnt is compared with the planned Task-Time
			represented through a factor of 100�s (Task-Time 20ms -> 200 = 200*100�s).
			If the tRootTask.ulCnt is exact 200, the modulo is == zero and exact
			20ms are over. This is repeated at tRootTask.ulCnt = 400, 600 etc. until the Reset
			of the counter at 50000.
			So within the 5 seconds the Task with 20ms Task-Time will appear 250 times.

			During this low priority Root Task e.G. the MSC-Detection is handeled!
			All 250ms a new Angle-Accelaration value will be calculated for use in Inertia-Correction!
			All 100ms the Diagnose LED at GPIO49 will be tooggled!

\fn			void RootTaskA1(void)
\author		K�ster, Ralf
\date		13.04.2012
\param[IN]	
\return		
\details	Changes:	
*/
void RootTaskA1(void) //
{
	Uint16 usState;
	tRootTask.ulCnt++;
	
	// tasks to do all 1ms
	if( (tRootTask.ulCnt % TSK_1MS) == 0 )
	{
		// actions, if errors are detected
		//DiagnosCheckStates();
		// Diagnosis and Version Update
		UpdateSystemInfo();

	}

	// tasks to do all 5ms, MSC Data-Aquisition, 
	if( (tRootTask.ulCnt % TSK_5MS) == 0 )
	{
		// MSC data Aquisition ongoing?
		if( *PtrMscState() == MSC_DATA_AQU )
		{
			// MSC data Aquisition is ready, when usState = TRUE
			usState = (Uint16)MscDatAq(&tMscData, *PtrAngleAbs(), *PtrCrTorqueRaw());
			if( usState == TRUE )
			{
				// semaphore for RootTask MscTblCalc() in main.c, data ok, start with analysation
				*PtrMscState() = MSC_START_ANALYSE;					
			}
			else
			{
				// otherwise nothing to do
				asm(" nop");
			}
		}
		//NOP for Breakpoint
		asm(" nop");
	}

	// tasks to do all 100ms
	if( (tRootTask.ulCnt % TSK_100MS) == 0 )
	{
		// LED all 100ms (blink blink!)
		if( (*PtrDsMotorCon()== MC_ADC_NODATA) )
		{
			//GpioDataRegs.GPBTOGGLE.bit.GPIO49 = 1;
			//NOP for Breakpoint
			asm(" nop");
		}
		else
		{
		}
	}

	// tasks to do all 250ms
	if( (tRootTask.ulCnt % TSK_250MS) == 0 )
	{
		//NOP for Breakpoint
		asm(" nop");
	}

	// tasks to do all 1000ms
	if( (tRootTask.ulCnt % TSK_1000MS) == 0 )
	{
		// Calculation of Angle Accelaration all 250ms
		//AngleAccelaration();

		
		if( (*PtrDsMotorCon() == PWM_OVCUR) )
		{
			//GpioDataRegs.GPBTOGGLE.bit.GPIO49 = 1;
			//NOP for Breakpoint
			asm(" nop");
		}
		else
		{
		}

		// after 1 sec FOC Initializing, but only at program start
		if( tRootTask.usSemaphor == 1)
		{
			//start FOC only, when IP and ADC without bugs
			if( (*PtrDsIP() == IP_NOERROR) && (*PtrDsMotorCon()== MC_NOERROR) )
			{
				//PWMEnable();
				//DELAY_US(60);
				// finally start the FOC
				//StartFOC();
				//DELAY_US(3000);
			}
			tRootTask.usSemaphor = tRootTask.usSemaphor +1;
		}
		else if( tRootTask.usSemaphor == 2)
		{
			//NOP for Breakpoint
			asm(" nop");
		}
	}

	// Reset counter without error in time
	if(tRootTask.ulCnt == 50000)
	{
		tRootTask.ulCnt = 0;
	}
	// give back the Control to Root-task
	PtrRootTask = &RootTaskA0;
}

// 
Uint16* PtrRootTskSem(void)
{
	return &tRootTask.usSemaphor;
}




#ifdef RAUS
unsigned int unChange = 0;
float32 fTime2 ;
void RootTaskA2(void) //** Toggle GPIO-2
{	

	if(unChange == 1)
	{
		SetCondTimer2(fTime2);
		unChange = 0;
	}
	//*PtrSsiRecComm() = TRUE;
	//go back to A0
	PtrRootTask = &RootTaskA0;
}
#endif

