
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "SpiA.h"
#include "Ip2000.h"
#include "GCIP2000.h"
//#include "hardware.h"
//#include "error_codes.h"
#include "angle.h"
#include "Timer.h"

/* Macro definitions ********************************************************/

/* Types ********************************************************************/

/* Local Prototypes *********************************************************/

/* Global variables *********************************************************/
//struct GCIP2000REG_CONT gcip2000;//! Struktur, welche alle IP-Register enthlt

/* Local variables **********************************************************/

/* Functions ****************************************************************/

//*****************************************************************************
/*! \brief Initialisierung der jeweiligen Gpiopins fr SPI 
 *
 *	Initialisierung der jeweiligen Gpiopins fr SPI 
 * 
 */
//*****************************************************************************
void InitSpiAGpio(void)
{
	EALLOW;

	// Init Pins			
    GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;   // Enable pull-up on GPIO16 (SPISIMOA)
    GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;   // Enable pull-up on GPIO17 (SPISOMIA)
    GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;   // Enable pull-up on GPIO18 (SPICLKA)
    GpioCtrlRegs.GPAPUD.bit.GPIO19 = 0;   // Enable pull-up on GPIO19 (SPISTEA)

    GpioCtrlRegs.GPAQSEL2.bit.GPIO16 = 3; // Asynch input GPIO16 (SPISIMOA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO17 = 3; // Asynch input GPIO17 (SPISOMIA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO18 = 3; // Asynch input GPIO18 (SPICLKA)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO19 = 3; // Asynch input GPIO19 (SPISTEA)

    GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 1; // Configure GPIO16 as SPISIMOA
    GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 1; // Configure GPIO17 as SPISOMIA
    GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 1; // Configure GPIO18 as SPICLKA
    GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 1; // Configure GPIO19 as SPISTEA

	EDIS;
	
	EALLOW;
	//  GPIO-77 = Reset IP2000
	GpioCtrlRegs.GPCMUX1.bit.GPIO77 = 0;	// 0=GPIO
	GpioCtrlRegs.GPCDIR.bit.GPIO77 = 1;		// 1=OUTput,  0=INput 
	//Reset an IP2000 auf L ber Transistor an TMS
	GpioDataRegs.GPCSET.bit.GPIO77 = 1;		// uncomment if --> Set High initially
	
	//GPIO5 = Triggereingang von IP2000, siehe DSP2833x_EPwm.c
	
	//GPIO11 = MCLK_IP2000 von ECAP4
	GpioCtrlRegs.GPAPUD.bit.GPIO11 = 1;		// Disable pull-up on GPIO11
	GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 3;	// Configure GPIO11 as ECAP4
	
	//Referenzsignalimpuls
	GpioCtrlRegs.GPBPUD.bit.GPIO52 = 0;		// Enable pull-up on GPIO52
	GpioCtrlRegs.GPBDIR.bit.GPIO52 = 0;		//Input
	GpioCtrlRegs.GPBQSEL2.bit.GPIO52 = 3;	//Asynchrone Datenannahme
	
	//Im DSP/Controller Mode des Ip gibt dieser einen Impuls aus,
	//wenn im Triggerregister ein Datum anliegt
	GpioCtrlRegs.GPBPUD.bit.GPIO50 = 0;		// Enable pull-up on GPIO50
	GpioCtrlRegs.GPBDIR.bit.GPIO50 = 0;		//Input
	GpioCtrlRegs.GPBQSEL2.bit.GPIO50 = 3;	//Asynchrone Datenannahme
	
	EDIS;
}

//*****************************************************************************
/*! \brief Initialisierung des SPI-Moduls fr die Kommunikation mit dem IP2000
 *
 *	author david.berthold
 *	
 *	Initialisierung des SPI-Moduls fr die Kommunikation mit dem IP2000, jedoch
 * 	fr das Senden und Empfangen ohne Interrupt
 */
//*****************************************************************************
void SpiAInit(void){

	// Initialize SPI registers
	SpiaRegs.SPICCR.bit.SPISWRESET=0;		// Reset SPI

	//Rising edge with delay, siehe Figure  1-4
	SpiaRegs.SPICCR.bit.SPICHAR = 0xF;		// 16 Bits
	SpiaRegs.SPICCR.bit.CLKPOLARITY = 0;	// Data is output on falling edge and
											// input on rising edge.
	SpiaRegs.SPICTL.bit.MASTER_SLAVE = 1;	// SPI is Master
	SpiaRegs.SPICTL.bit.CLK_PHASE = 1;		// SPICLK signal delayed by one half-cycle;
											// polarity determined by the CLOCK POLARITY bit

	SpiaRegs.SPICTL.bit.TALK = 1;			// Enable Transmission with CS

	// Baud rate: For SPIBRR = 3 to 127: SPI Baud Rate = LSPCLK/(SPIBRR + 1)
	//SpiaRegs.SPIBRR = 4;					//15MHz, scheint zu gehen
	SpiaRegs.SPIBRR = 6;					//10,71MHz, zur Sicherheit

	SpiaRegs.SPIFFTX.bit.SPIRST = 1;		// SPI FIFO can resume transmit or receive
	SpiaRegs.SPIFFTX.bit.SPIFFENA = 1;		// SPI FIFO enhancements are enabled
	//SpiaRegs.SPIFFTX.bit.TXFFIENA = 0;		// TX FIFO interrupt based on TXFFIVL match
											// (less than or equal to) will be enabled.
	//SpiaRegs.SPIFFTX.bit.TXFFIL = 0x1;		// 1 Word to send Int.


	//SpiaRegs.SPIFFRX.bit.RXFFIENA = 1;		// RX FIFO interrupt based on RXFFIL match
											// (greater than or equal to) will be enabled.
	//SpiaRegs.SPIFFRX.bit.RXFFIL = 0x1;		// 1 Word


	SpiaRegs.SPIPRI.bit.FREE = 1;			// Free run, continue SPI operation regardless
											// of suspend or when the suspend occurred.

	SpiaRegs.SPICCR.bit.SPISWRESET=1;		// Enable SPI

	SpiaRegs.SPIFFTX.bit.TXFIFO=1;			// Transmit FIFO reset
	SpiaRegs.SPIFFRX.bit.RXFIFORESET=1;		// Receive FIFO reset

}

//*****************************************************************************
/*! \brief Reset des Sende- und Empfangspuffers
 *
 *	author david.berthold
 */
//*****************************************************************************
void SpiAFifoReset(void){
	//Reset FIFOs
	SpiaRegs.SPIFFTX.bit.TXFIFO = 0;
	SpiaRegs.SPIFFRX.bit.RXFIFORESET = 0;
	
	asm("\n nop;\n");
	SpiaRegs.SPIFFTX.bit.TXFIFO = 1;
	SpiaRegs.SPIFFRX.bit.RXFIFORESET = 1;
}

//*****************************************************************************
/*! \brief Umkonfiguration des SPI-Moduls fr die interruptgesteuerte 
 * 	Kommunikation mit dem IP2000
 *
 *	author david.berthold
 *	
 *	Umkonfiguration des SPI-Moduls fr die interruptgesteuerte Kommunikation 
 * 	mit dem IP2000
 */
//*****************************************************************************
void SpiAInitInterruptSendRecv(void){

	// Initialize SPI registers
	SpiaRegs.SPICCR.bit.SPISWRESET=0;		// Reset SPI

	//SpiaRegs.SPIFFTX.bit.TXFFIENA = 0;	// TX FIFO interrupt based on TXFFIVL match
											// (less than or equal to) will be enabled.
	//SpiaRegs.SPIFFTX.bit.TXFFIL = 0x1;	// 1 Word to send Int.


	SpiaRegs.SPIFFRX.bit.RXFFIENA = 1;		// RX FIFO interrupt based on RXFFIL match
											// (greater than or equal to) will be enabled.
	SpiaRegs.SPIFFRX.bit.RXFFIL = IP2000_SEND_ANZAHL;		// 4 Word
	
	//Delay von min. 250ns (4/f_OSZ + 20ns + 70ns = 4/25MHz + 20ns + 70ns = 160 + 20 + 70)
	//zwischen dem gesendeten laut Datenblatt
	//SpiaRegs.SPIFFCT.bit.TXDLY = 3;			//Anzahl der SPI-Takte, wieviel Delay zw. Puffer und Senderegister
	SpiaRegs.SPIFFCT.bit.TXDLY = (unsigned int)(250e-9 * LOSP_FRQ / (SpiaRegs.SPIBRR + 1)) + 1;
	//TXDLY = SPITAKT * Delay
	
	SpiaRegs.SPIFFRX.bit.RXFFINTCLR = 1;


	SpiaRegs.SPICCR.bit.SPISWRESET=1;		// Enable SPI
	SpiAFifoReset();
}

//*****************************************************************************
/*! \brief Umkonfiguration des SPI-Moduls fr die normale Kommunikation mit dem
 * 	IP2000
 *
 *	author david.berthold
 *	
 *	Umkonfiguration des SPI-Moduls fr die normale Kommunikation mit dem IP2000
 * 	ohne interruptgesteuertes Senden/Empfangen
 */
//*****************************************************************************
void SpiADeinitInterruptSendRecv(void){
	//Warten bis letztes Byte versendet
	if(SpiaRegs.SPIFFTX.bit.TXFFST != 0){
		unsigned int received_data = SpiaRegs.SPIFFRX.bit.RXFFST;
		//Warten bis letztes fertig mit senden
		while(received_data == SpiaRegs.SPIFFRX.bit.RXFFST);
	}
	SpiaRegs.SPICCR.bit.SPISWRESET=0;		// Reset SPI

	SpiaRegs.SPIFFRX.bit.RXFFIENA = 0;		// RX FIFO interrupt based on RXFFIL match
											// (greater than or equal to) will be enabled.
	SpiaRegs.SPIFFRX.bit.RXFFINTCLR = 1;

	SpiaRegs.SPICCR.bit.SPISWRESET=1;		// Enable SPI
	SpiAFifoReset();
}



//*****************************************************************************
/*! \brief Lesen eines 32Bit Wertes vom IP2000
 *
 * 	\param adr Adresse des zu lesenden Registers
 *
 *	author david.berthold
 * 	Achtung: Diese Funktion funktioniert nur, wenn das interruptgesteuerte 
 * 	Senden/Empfangen deaktiviert ist.
 *
 * 	\return		Inhalt des gelesenen Registers
 */
//*****************************************************************************
unsigned long SpiARead32(unsigned char adr)
{
	unsigned int recdata;
	
	while(SpiaRegs.SPIFFTX.bit.TXFFST != 0);		//warten auf Ende der letzten Transmission;
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	
	SpiAFifoReset();
	//Wert in Puffer schreiben
	SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|((unsigned int)(adr));	// Adresse
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	recdata = SpiaRegs.SPIRXBUF;
	
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_RD1<<12));	// Adresse 32 Bit
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	recdata = SpiaRegs.SPIRXBUF;
	
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiaRegs.SPITXBUF = (GCIP_OPC_NOP<<12);			// Dummy
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	
	return (((unsigned long)(SpiaRegs.SPIRXBUF)<<16) | ((unsigned long)recdata));
}

//*****************************************************************************
/*! \brief Lesen eines 16Bit Wertes vom IP2000
 *
 * 	\param adr Adresse des zu lesenden Registers
 *
 *	author david.berthold
 * 	Achtung: Diese Funktion funktioniert nur, wenn das interruptgesteuerte 
 * 	Senden/Empfangen deaktiviert ist.
 *
 * 	\return		Inhalt des gelesenen Registers
 */
//*****************************************************************************
unsigned int SpiARead16(unsigned char adr)
{
	volatile unsigned int recdata;
	
	while(SpiaRegs.SPIFFTX.bit.TXFFST != 0);		//warten auf Ende der letzten Transmission;
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	
	SpiAFifoReset();
	//Wert in Puffer schreiben
	SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|((unsigned int)(adr));	// Adresse
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	recdata = SpiaRegs.SPIRXBUF;
	
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiaRegs.SPITXBUF = (GCIP_OPC_NOP<<12);			// Dummy
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	
	return SpiaRegs.SPIRXBUF;
}

//*****************************************************************************
/*! \brief Schreiben eines 8Bit Wertes zum IP2000
 *
 *	\param data Zu schreibende Daten
 * 	\param adr Adresse des zu lesenden Registers
 *
 *	author david.berthold
 * 	Schreiben eines 8Bit Registers, allerdings nur schreiben der Werte in den
 * 	Sendepuffer.
 * 	Achtung: Diese Funktion funktioniert nur, wenn das interruptgesteuerte 
 * 	Senden/Empfangen deaktiviert ist.
 */
//*****************************************************************************
void SpiAWrite8Buffer(unsigned char data, unsigned char adr){
	SpiaRegs.SPITXBUF = (GCIP_OPC_WRA<<12)|(GCIP_OPC_HWA<<8)|(unsigned int)(adr);	// Adresse
	SpiaRegs.SPITXBUF = (GCIP_OPC_WRD<<12)|(GCIP_OPC_HWA<<8)|(unsigned int)(data);	// Data
}

//*****************************************************************************
/*! \brief Schreiben eines 8Bit Wertes zum IP2000
 *
 *	\param data Zu schreibende Daten
 * 	\param adr Adresse des zu lesenden Registers
 *
 *	author david.berthold
 * 	Achtung: Diese Funktion funktioniert nur, wenn das interruptgesteuerte 
 * 	Senden/Empfangen deaktiviert ist.
 */
//*****************************************************************************
void SpiAWrite8(unsigned char data, unsigned char adr)
{
	volatile unsigned long recdata;
	
	while(SpiaRegs.SPIFFTX.bit.TXFFST != 0);		//warten auf Ende der letzten Transmission;
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiAFifoReset();
	
	SpiaRegs.SPITXBUF = (GCIP_OPC_WRA<<12)|(GCIP_OPC_HWA<<8)|(unsigned int)(adr);	// Adresse
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	recdata = SpiaRegs.SPIRXBUF;					// Read data
	
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiaRegs.SPITXBUF = (GCIP_OPC_WRD<<12)|(GCIP_OPC_HWA<<8)|(unsigned int)(data);	// Data
	while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
	recdata = SpiaRegs.SPIRXBUF;					// Read data
}

//*****************************************************************************
/*! \brief Schreiben eines 32Bit Wertes zum IP2000
 *
 *	\param data Zu schreibende Daten
 * 	\param adr Adresse des zu lesenden Registers
 *
 *	author david.berthold
 * 	Achtung: Diese Funktion funktioniert nur, wenn das interruptgesteuerte 
 * 	Senden/Empfangen deaktiviert ist.
 */
//*****************************************************************************
void SpiAWrite32(unsigned long data, unsigned char adr)
{
	unsigned char i = 0;
	//Damit die Daten aus dem RXFIFO gelesen werden
	volatile unsigned int recdata;
	
	while(SpiaRegs.SPIFFTX.bit.TXFFST != 0);		//warten auf Ende der letzten Transmission;
	DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
	//Warten bis IP2000 bereit, Wartezeit nach CS steigend
	while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
	SpiAFifoReset();


	for(i=0; i<4; i++){
		// Adresse 8 Bit
		SpiaRegs.SPITXBUF = (GCIP_OPC_WRA<<12)|(GCIP_OPC_HWA<<8)|(unsigned int)(adr);
		while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission, weil sonst CS Low bleibt
		recdata = SpiaRegs.SPIRXBUF;					// Read data
		
		DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
		//Warten bis IP2000 bereit, Wartezeit nach CS steigend
		while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);

		// Daten 8 Bit
		SpiaRegs.SPITXBUF = (GCIP_OPC_WRD<<12)|(GCIP_OPC_HWA<<8)|(data&0xFF);
		while(SpiaRegs.SPIFFRX.bit.RXFFST == 0);		//warten auf Ende der Transmission;
		recdata = SpiaRegs.SPIRXBUF;					// Read data
		
		DELAY_US(1);//Verzgerungszeit CS steigend bis nWAIT(L) aktiv
		//Warten bis IP2000 bereit, Wartezeit nach CS steigend
		while(GpioDataRegs.GPADAT.bit.GPIO17 == 0);
		//Die nchsten 8Bit werden mit der inc. Adresse geschrieben
		adr++;
		data>>=8;		
	}

}
//*****************************************************************************
/*! @}
 *	Close the Doxygen group.
 */
//*****************************************************************************
