/*!
\file	Test.c
\brief	Drehzahlberechnung / Calculation Omega

\author	Koester
\date	17.10.2012

\details
 changes:	

*/

/* Include files ************************************************************/
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "mth.h"
#include "Test.h"
#include "measmode.h"
#include "IntSvc.h"

#ifdef RAUS

#define NONEMODE	0

struct
{
	int					nErr;
	unsigned short 		usMode;
	unsigned short		usTestCounter;
} stcTest;

float32 fSinus, fWert = 0.0;


void Test_Init(void)
{
	stcTest.usMode = NONEMODE;
	/* initialisieren, aber zur Zeit ungenutzt */	
	stcTest.nErr = 0;
	stcTest.usTestCounter = 0;
}


/*!
\fn		void Test_Irq()
\brief  Interruptroutine Timer-1 fr Diagnosezwecke DSP-1 Control
		Aufruf zur Zeit alle 100ms!
		Funktion wie eine IDLE-TASK!

\param[in]

*/

void Test_Irq(void)
{
	switch (stcTest.usMode)
	{
		/* keine Ausgabe an AC-Ampl. */
		case NONEMODE:
			//NOP for Breakpoint
			asm(" nop");
			fWert = fWert + 0.001;
			//fSinus = pow( sin( fWert), 2.0);
		break;

		default:
		//NOP for Breakpoint
		asm(" nop");
		break;
	}

	//NOP for Breakpoint
	asm(" nop");
}



/***************************************************************************
 * Function:     TestDaten (float p_fData, unsigned int p_unAction)
 * Description:   Sammel, lschen und ausgeben ber RS232 von Testdaten
 *                im float Format. 100 Float Werte knnen gesammelt und
 *                ber RS232 ausgelesen werden.
 *                Prozedur Daten sammeln:
 *                 // Testdaten schreiben, wenn noch Platz im Array
 *                if( TestDaten(0.0, PROOF)==TRUE)
 *                {
 *                   // Testdaten speichern
 *                   TestDaten (tLiftDat.fFnRaw, SET);
 *                }
 *                Prozedur Daten auslesen: (Kommando 'T TD')
 *                 //
 *                TestDaten (0.0, PRINT);
 * 
 * 
 * Parameters:   
 * Return value: 
 * Comments:     
 ***************************************************************************/

#define MAXDATA 100
// data for testdata handling
typedef struct
{
	float32	fV[MAXDATA];				// Testmodus Lift obere Position in [m]
	Uint16 unI, unJ;
}t_stcLiftTest;
t_stcLiftTest tTd;


Uint16 TestDaten (float32 p_fData, Uint16 p_unAction)
{
	switch (p_unAction)
	{
      // check if array full
      case PROOF:
         if(tTd.unI > MAXDATA-1)
         {
            return(FALSE);
         }
         else
         {
            return(TRUE); 
         }
      // store value
		case SET:
			// bergebenes Datum speichern
			tTd.fV[tTd.unI] = p_fData;
			tTd.unI++;
			return(TRUE);
		// delete array and reset counters
		case CLEAR:
         for(tTd.unI=0; tTd.unI < MAXDATA; tTd.unI++)
         {
            tTd.fV[tTd.unI] = 0.0;
         }
			tTd.unI = 0;
			tTd.unJ = 0;
			return(TRUE);
		// send data to the called channel
		case SEND:
		// 27.03.13 send data to the channel-6
		*PtrMeasTestF6() = tTd.fV[tTd.unJ];
		tTd.unJ++;
		if( tTd.unJ > MAXDATA-1 )
		{
			tTd.unI = 0;
			tTd.unJ = 0;
		}
		return(TRUE);
		//
		default:
			return(FALSE);
		
	}

}




float32 CreateTestTorqueData(float32 fFreq, float32 fAmpl)
{
	float32 fAngStep, fTrq;
	Uint16 k;
	static Uint16 i;
	// Anz. Winkelschritte aus f und tabt. berechnen
	k = 1e06*(1/(fFreq* *PtrSamplingTime() ));
	if( (i>k) || (i<1) ) i=1;
	// daraus Winkelinkrement
	fAngStep = (cf2Pi / k);
	// Strpegel berechen
	fTrq = 	fAmpl * sin(i*fAngStep);
	i++;
	return fTrq;
}
#endif
