/*!
\file	Timer.c
\brief	Control Timer TMS320F28335 
		
\author	Koester / TI 
\date	11.10.2011

\details
	changes:

*/


/* Include files ************************************************************/
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "Timer.h"
#include "Ip2000.h"
#include "IntSvc.h"

/* Macro definitions ********************************************************/

/* Types ********************************************************************/

/* Local Prototypes *********************************************************/

			 
/* Global variables *********************************************************/

/* Datenstrukturen fr Timer-Interupte, vornehmlich fr Diagnose */
struct CPUTIMER_VARS CpuTimer0;
// When using DSP BIOS & other RTOS, comment out CPU Timer 2 code.
struct CPUTIMER_VARS CpuTimer1;
struct CPUTIMER_VARS CpuTimer2;

//---------------------------------------------------------------------------
// InitCpuTimers:
//---------------------------------------------------------------------------
// This function initializes all three CPU timers to a known state.
//
void InitCpuTimers(void)
{
    // CPU Timer 0
    // Initialize address pointers to respective timer registers:
    CpuTimer0.RegsAddr = &CpuTimer0Regs;
    // Initialize timer period to maximum:
    CpuTimer0Regs.PRD.all  = 0xFFFFFFFF;
    // Initialize pre-scale counter to divide by 1 (SYSCLKOUT):
    CpuTimer0Regs.TPR.all  = 0;
    CpuTimer0Regs.TPRH.all = 0;
    // Make sure timer is stopped:
    CpuTimer0Regs.TCR.bit.TSS = 1;
    // Reload all counter register with period value:
    CpuTimer0Regs.TCR.bit.TRB = 1;
    // Reset interrupt counters:
    CpuTimer0.InterruptCount = 0;

// CpuTimer2 is reserved for DSP BIOS & other RTOS
// Do not use this timer if you ever plan on integrating
// DSP-BIOS or another realtime OS.

    // Initialize address pointers to respective timer registers:
    CpuTimer1.RegsAddr = &CpuTimer1Regs;
    CpuTimer2.RegsAddr = &CpuTimer2Regs;
    // Initialize timer period to maximum:
    CpuTimer1Regs.PRD.all  = 0xFFFFFFFF;
    CpuTimer2Regs.PRD.all  = 0xFFFFFFFF;
    // Make sure timers are stopped:
    CpuTimer1Regs.TCR.bit.TSS = 1;
    CpuTimer2Regs.TCR.bit.TSS = 1;
    // Reload all counter register with period value:
    CpuTimer1Regs.TCR.bit.TRB = 1;
    CpuTimer2Regs.TCR.bit.TRB = 1;
    // Reset interrupt counters:
    CpuTimer1.InterruptCount = 0;
    CpuTimer2.InterruptCount = 0;

}

//---------------------------------------------------------------------------
// ConfigCpuTimer:
//---------------------------------------------------------------------------
// This function initializes the selected timer to the period specified
// by the "Freq" and "Period" parameters. The "Freq" is entered as "MHz"
// and the period in "uSeconds". The timer is held in the stopped state
// after configuration.
//
void ConfigCpuTimer(struct CPUTIMER_VARS *Timer, float Freq, float Period)
{
    Uint32  temp;

    // Initialize timer period:
    Timer->CPUFreqInMHz = Freq;
    Timer->PeriodInUSec = Period;
    temp = (long) (Freq * Period);
    Timer->RegsAddr->PRD.all = temp;

    // Set pre-scale counter to divide by 1 (SYSCLKOUT):
    Timer->RegsAddr->TPR.all  = 0;
    Timer->RegsAddr->TPRH.all  = 0;

    // Initialize timer control register:
    Timer->RegsAddr->TCR.bit.TSS = 1;      // 1 = Stop timer, 0 = Start/Restart Timer
    Timer->RegsAddr->TCR.bit.TRB = 1;      // 1 = reload timer
    Timer->RegsAddr->TCR.bit.SOFT = 1;
    Timer->RegsAddr->TCR.bit.FREE = 1;     // Timer Free Run
    Timer->RegsAddr->TCR.bit.TIE = 1;      // 0 = Disable/ 1 = Enable Timer Interrupt

    // Reset interrupt counter:
    Timer->InterruptCount = 0;
}

//===========================================================================
// End of file.
//===========================================================================






/*!
Title:		\brief Init and Start Timer0
Function:	\fn Timer0InitStart(void)
Name:		\author Kster, Ralf
Date:		\date 04.12.2008
Description:

Parameters:	\param[IN] void
Return:			\return void

Changes:	
*/
void Timer0InitStart(float32 p_fTime)
{
	// Configure CPU-Timer 0
	// 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer0, CPU_SPEED, p_fTime);
	// Start Timer0
	CpuTimer0Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0

}


/*!
\fn		SetCondTimer0(float p_fTime)
\brief  Einstellen Zeitperiode Timer0

\param[IN]	p_fTime, Zeit in s

*/
void SetCondTimer0(float32 p_fTime) 
{
	// Timer 0 Stop
	StopCpuTimer0();	
	// Configure CPU-Timer-0, 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer0, CPU_SPEED, p_fTime);
	// Start again
	StartCpuTimer0();
}




/*!
\fn		void Timer0Start(void)
\brief  

*/
void Timer0Start(void) 
{
	StartCpuTimer0();
}



/*!
\fn		Timer0Stop(void)
\brief  Anhalten Timer0

\param[IN]	void

*/
void Timer0Stop(void)
{
	StopCpuTimer0();
}



////////////////////////////////////////////////////////////////
///	\brief		Init of Current-Control-Timer
///	\ingroup	Timer
///	\param		void
/// \return		void
////////////////////////////////////////////////////////////////

void Timer1InitStart(float32 p_fTime)
{
	// Configure CPU-Timer 0
	// 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer1, CPU_SPEED, p_fTime);
	// Start Timer0
	CpuTimer1Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0

}


/*!
\fn		SetCondTimer1(float p_fTime)
\brief  Einstellen Zeitperiode Timer0

\param[IN]	p_fTime, Zeit in s

*/
void SetCondTimer1(float32 p_fTime) 
{
	// Timer-1 Stop
	StopCpuTimer1();	
	// Configure CPU-Timer-1, 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer1, CPU_SPEED, p_fTime);
	// Start again
	StartCpuTimer1();
}




////////////////////////////////////////////////////////////////
///	\brief		Stop Idle-Timer
///	\ingroup	Timer
///	\param		void
/// \return		void
////////////////////////////////////////////////////////////////
void Timer1Stop(void)
{
	StopCpuTimer1();
}

/*!
\fn		void Timer1Start(void)
\brief  

*/
void Timer1Start(void) 
{
	StartCpuTimer1();
}




//==================================================================================
//	Timer-2 fr Root-Task, ohne Interrupt (wird gepollt)
//==================================================================================

/*!
\fn		Timer2InitStart(void)
\brief  Initialization and Start Timer-2 (use for Root Task)
\param[IN]	void
\return		void

*/
void Timer2InitStart(float32 p_fTime)
{
	// Configure CPU-Timer 1, 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer2, CPU_SPEED, p_fTime);
	// Start Timer2
	CpuTimer2Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0
}

/*!
\fn		SetCondTimer2(float p_fTime)
\brief  Einstellen Zeitperiode Timer0

\param[IN]	p_fTime, Zeit in s

*/
void SetCondTimer2(float32 p_fTime) 
{
	// Timer-2 Stop
	StopCpuTimer2();	
	// Configure CPU-Timer 0, 150MHz CPU Freq, 1 second Period (in uSeconds)
	ConfigCpuTimer(&CpuTimer2, CPU_SPEED, p_fTime);
	// Start again
	StartCpuTimer2();
}


/*!
\fn		Timer2InitStart(void)
\brief  Initialization and Start Timer-2 (use for Root Task)
\param[IN]	void
\return		void

*/
void Timer2Stop(void)
{
	StopCpuTimer2();
}


/*!
\fn		DELAY_US(unsigned long ulTime)
\brief  makes a delay of defined sec

\param[IN]	unsigned long ulTime

\return		void

*/
void DELAY_US(unsigned long ulTime)
{
    long      i,j=0,k;
	k = ulTime * 50;
    for (i = 0; i < k; i++)
    {
    	j = j+i;
		//NOP for Breakpoint
		asm(" nop");

    }
}
